/*
 * Frequency-response matrix object
 *
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
#ifndef _MATRIX_H_
#define _MATRIX_H_

#include <stdio.h>

class FullyFunctional {
private:
    unsigned char base;
    int dye;
    int filter;
    float dye_frequency;
    float filter_frequency;
public:
    FullyFunctional(unsigned char abase, int adye = -1, int afilter = -1);
    FullyFunctional(const FullyFunctional &temp);
    unsigned char get_base() { return base; }
    int get_dye() { return dye; }
    virtual ~FullyFunctional();
};

class ResponseMatrix {
private:
    float *matrix;
    int ndim;
    FullyFunctional **channels;
    float *exposure_times;
    float *power;
protected:
    virtual void reset_matrix();
    virtual void find_reordering(unsigned char *bases, int n, int *index);

public:
    ResponseMatrix(int n);
    virtual ~ResponseMatrix();
    virtual void set_matrix(float *matrix, int n, FullyFunctional **ffns,
                            float *exposures = NULL);
    virtual void set_matrix(float *matrix, int n = 0,
                            unsigned char *bases = NULL,
                            float *exposures = NULL);
    virtual int load(const char filename[]);
    virtual float *get_matrix(int n = 0, unsigned char *bases = NULL,
                              float *exposure = NULL);
    virtual float *get_inverse_matrix(int n = 0, unsigned char *bases = NULL,
                                      float *exposure = NULL);
    virtual void set_power_correction(float *power = NULL, int n = 0);
    virtual void apply(float *input, float *output, int ndata = 1,
                       int n = 0, unsigned char *bases = NULL);
    virtual void apply_inverse(float *input, float *output, int ndata = 1,
                               int n = 0, unsigned char *bases = NULL);
    virtual unsigned char get_base(int n) { return channels[n]->get_base(); }
    virtual int getSize() { return ndim; };
};

#endif //_MATRIX_H_
