//  Time-stamp: <26 Jun 2006 17:07 BST Logger.h km>
/*
 * Logger class.
 *
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#ifndef _LOGGER_H
#define _LOGGER_H

#include <string>
#include <vector>
#include <fstream>
#include <iostream>

const int LOGGING_LEVEL_SEVERE  = 7;
const int LOGGING_LEVEL_WARNING = 6;
const int LOGGING_LEVEL_INFO    = 5;
const int LOGGING_LEVEL_CONFIG  = 4;
const int LOGGING_LEVEL_FINE    = 3;
const int LOGGING_LEVEL_FINER   = 2;
const int LOGGING_LEVEL_FINEST  = 1;

using namespace std;

class LogHandler {
protected:
    int min_level;
    string source;
public:
    LogHandler() : min_level(LOGGING_LEVEL_INFO) {}
    virtual ~LogHandler() {};
    virtual void set_level(int level = 0) { min_level = level; }
    virtual void message(int level, const char message[]) 
    { if (level >= min_level) cout << message << std::endl; }
    virtual void set_source(const char src[]) { source = src; }
};

class FileLogHandler : public LogHandler {
    std::vector<char *> event_list;
    std::string filename;
    std::ofstream *file_ptr;
    char *format_message(int level, const char message[]);
    int line_buffer_size;
public:
    FileLogHandler(const char fn[]) : LogHandler(), line_buffer_size(100)
        { filename = fn; }
    virtual ~FileLogHandler() {  flush(); }
    virtual void message(int level, const char message[]);
    virtual void flush();
};

class Logger {
    std::vector<LogHandler *> handlers;
    string source;
private:
    void send_message(int level, const char message[]);
public:
    Logger();
    Logger(const char fn[]);
    virtual ~Logger();
    void config(const char message[]);
    void info(const char  message[]);
    void warning(const char message[]);
    void error(const char message[]);
    void flush();
    void set_source(const char src[]);
    void add_handler(LogHandler *handler);
};

#endif /* _LOGGER_H */
