#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fstream>
#include <iostream>
#include <vector>
#include <deque>
#include <string>
#include <cmath>
#include <algorithm>
#include <map>
#include <dirent.h>
#include <pthread.h>

#include <sys/resource.h>

#include "GlobalUtilities.h"
#include "ELAND_outer.h"

/*
PROJECT: ELAND (Efficient Local Alignment of Nucleotide Data)
MODULE: ELAND_outer.cpp
AUTHOR: A. J. Cox

Copyright (c) 2003-2006 Solexa Limited. All rights reserved.
This source file is covered by the "Solexa Public Source License"
agreement and bound by the terms therein.
*/

// Identifier to embed in source and binary files
static char* eland_outer_cpp_id="@(#) $Id: ELAND_outer.cpp,v 1.3 2006/10/06 14:45:46 tc Exp $";

// ELAND_outer.cpp

// Contains interface functionality from ELAND
// - specifically, stuff to do with scoring and storage of alignments


SuffixScoreTable::SuffixScoreTable() 
{
  int offset(0);

  buildScoreTable( scoreFragD_, fragLengthD, offset );
  offset+=fragLengthD;
  buildScoreTable( scoreFragC_, fragLengthC, offset );
  offset+=fragLengthC;
  buildScoreTable( scoreFragB_, fragLengthB, offset );
  offset+=fragLengthB;
  buildScoreTable( scoreFragA_, fragLengthA, offset );

} // ~SuffixScoreTable::SuffixScoreTable( int numBits ) 

void SuffixScoreTable::buildScoreTable
( vector<FragmentErrorType>& score, const int fragLength, const int offset )
{
  const Word numFrags( 1<<(fragLength<<1) );
  Word test, result;

  FragmentErrorType thisScore;
  int numErrors;
  score.resize(numFrags);
  for ( Word i(0); i<numFrags; i++ )
  {
    test = ((Word)0x3) << (numBitsPerBase*(fragLength-1));
    thisScore=0;
    numErrors=0;
    for (int j(1); j<=fragLength; j++, test>>=numBitsPerBase )
    {
      if ((result=(i&test))!=0)
      {
        ++numErrors;

        if (numErrors<=2)
        {
          if (numErrors==2)
            thisScore<<=errorBits__;
        
          thisScore|= (j+offset); 
 
          thisScore
            |= ((result>>(numBitsPerBase*(fragLength-j)))<<errorPosBits__);


          //            |= ((result>>(numBitsPerBase*(fragLength-1)))<<errorPosBits__);
          //	thisScore+=incrementDist;
          //	if ((thisScore&errorMask1)==0)
          //  thisScore+=j+offset;
          //	else if ((thisScore&errorMask2)==0)
          //	  thisScore+=((j+offset)<<errorBits);
        } // ~if
        else thisScore=moreThanTwoErrors__;

      } // ~if
    } // ~for j
    //    if (thisScore>=moreThanTwoErrors) thisScore &= distanceMask;
    score[i]=thisScore;
    //    printWord(i, fragLength);
    //   cout << " " << (thisScore&errorPosMask1__) << " "<< ((thisScore&errorPosMask2__)>>8) << endl;

    //        cout << " " << (thisScore>>(2*errorBits)) 
    // << " " << (thisScore&errorMask1) 
    // << " " << ((thisScore&errorMask2)>>errorBits) << endl; 
  } // ~for i

} // ~void SuffixScoreTable::buildScoreTable






MatchTable* MatchTable::getMatchTable( const char* outputFileName )
{
  const char suffixName[] =".vmf";
  const char* p(strstr(outputFileName, suffixName));
  if ((p==NULL)
      ||(p!=outputFileName
	 +strlen(outputFileName)
	 -strlen(suffixName)))
  {
    cerr << "Will output assembly format match details to file " 
	 << outputFileName << endl;
    return new MatchTableAssembly(outputFileName);
  } // ~if
  else
  { 
    cerr << "Will output verbose format match details to file " 
	 << outputFileName << endl;
    return new MatchTableVerbose(outputFileName);
  } // ~else
} // ~static MatchTable* MatchTable::getMatchTable


// isInterested__: hash table uses this function to ask 
// if it should hash an oligo
bool MatchTable::isInterested__( int oligoNum, int PASS, bool hasNs )
{
  if (PASS==0)
  {
    // if no repeat or QC info set, interested in oligo
    if (size()==0) return true;
    if ((hasQualityFailed__(oligoNum))
        ||(isRepeatMasked__(oligoNum)))
      return false;
    else
      return true;
  } // ~if
  else
  {
    if ((hasQualityFailed__(oligoNum))
        ||(isRepeatMasked__(oligoNum)))
      return false;
    if (!hasNs)
    {
      //      if (matchPosition_[oligoNum]==noMatch)
      //   return true;
      if (  ((matchType_[oligoNum].errorType&0x3)==0)
            && (matchType_[oligoNum].r[0]>1))
      {
        //        cout << ((int)matchType_[oligoNum].r[0]) << " " << ((int)matchType_[oligoNum].r[1]) << " " << ((int)matchType_[oligoNum].r[2]) << endl;

        return false;
      } // ~if
      else
      {
        return true;
      } // ~else
    } // ~if
    else
    {
      return true;
    } // ~else

  } // ~else
} // ~MatchTable::wantMatch__( int PASS, bool hasNs )


// addMatch: process a match
// oligoNum - number of the oligo found
// matchPos - where it was found in the genome
// matchType - number and position of any errors in the match
// passNumber - which pass of the genome the match was found on
void MatchTable::addMatch( const TableEntry<useSplitPrefix>& entry, 
			   const Word thisMask,
			   const MatchPosition matchPos,
			   const FragmentErrorType errorLow,
			   const FragmentErrorType errorHigh )
			   // const int numErrorsz
{ 
  //  cout << "addMatch: " << " " <<  matchPos-16777216 << " " 
  //     << errorLow << " " << errorHigh << endl;


  const int numErrors
    ( (errorLow>oneError__)+(errorLow>noErrors__)
      +(errorHigh>oneError__)+(errorHigh>noErrors__));

  //   cout << "am " << errorLow << " " << errorHigh << " " << noErrors__ << " " << oneError__ << endl;
  //   cout << "am " << matchPos-16777216 << " " << numErrors << " " << (errorLow&errorPosMask1__) << " "
  // << ((errorLow&errorPosMask2__)>>8) << " "
  // << (errorHigh&errorPosMask1__) << " "
  //  << ((errorHigh&errorPosMask2__)>>8) << endl;



  //  assert(numErrors==numErrorsz);
  assert((numErrors>=0)&&(numErrors<=2));

  // For oligos with no Ns:
  // On partition 0, interested in 0, 1 and 2 error matches
  // On partition 1, interested in 0 or 1 error matches
  // On partitions 2 to 5, interested in 2 error matches only

  const OligoNumber oligoNum(entry.position
			     &((~isReverseOligo)&splitPrefixMaskLow));
  const bool reverseFlag((entry.position&isReverseOligo)!=0);
  //    oligoNum&=(~isReverseOligo);
    //    cout << 'Y' << endl;

    //    if (matchPosition_[oligoNum] != matchPos )
    //    {
#if (NUM_THREADS>1)
  pthread_mutex_lock(&mutex_);
#endif
      if (    ( numErrors < (matchType_[oligoNum].errorType&0x3))
	    || (matchPosition_[oligoNum]==noMatch))
      {
	// Then match is better than any match found so far
	if ( numErrors==2 )
	{
	  // store positions of both errors

          //	  matchType_[oligoNum].r[0] 
          //   = errorLow&errorMask1;
          //  matchType_[oligoNum].r[1] 
          //   = errorHigh&errorMask1;
          // TC 09.08.05 - above lines have a bug - they don't store both
          // match positions if both matches occur in the same fragment

	  matchType_[oligoNum].r[0] 
	    = errorLow&errorInfoMask1__;
          if (matchType_[oligoNum].r[0]==0)
          { // then both errors must be in errorHigh
            matchType_[oligoNum].r[0] 
              = errorHigh&errorInfoMask1__;
            matchType_[oligoNum].r[1] 
              = (errorHigh&errorInfoMask2__)>>errorBits__;
          } // ~if
          else
          { //  assume 2nd error is in upper bits of errorHigh...

            matchType_[oligoNum].r[1] 
              = (errorLow&errorInfoMask2__)>>errorBits__;

            // ... but if it's zero ...
            if (matchType_[oligoNum].r[1]==0)
            { // ... 2nd error must instead be in lower bits of errorLow
              matchType_[oligoNum].r[1] 
                = (errorHigh&errorInfoMask1__);
            } // ~if
          } // ~else

          assert(matchType_[oligoNum].r[0] !=0);
          assert(matchType_[oligoNum].r[1] !=0);
          assert((matchType_[oligoNum].r[0]&ucharErrorTypeMask) !=0);
          assert((matchType_[oligoNum].r[1]&ucharErrorTypeMask) !=0);


	} // ~if
	else if( numErrors==1 )
	{
	  // store position of single error
	  matchType_[oligoNum].r[0] 
	    = (errorLow|errorHigh)&errorInfoMask1__;
          //	    = (errorLow|errorHigh)&errorMask1;
	}
	else if( numErrors==0 )
	{
	  // if only 2 error matches found before, set r[1] to zero
	  // to signify no 1 error matches found so far
	  // else leave r[1] alone! - otherwise we lose all 1 error
	  // matches counted thus far (bug fix over previous version)
	  // TC 05.03.04
	  if ((matchType_[oligoNum].errorType&0x3)==2)
	    matchType_[oligoNum].r[1]=0;
	}
	//	else assert(1==0);

	// this is the first occurrence of this match type
	matchType_[oligoNum].r[numErrors]=1;

	// store its position
	matchPosition_[oligoNum] = matchPos;

	Word nInfo(0);
	if (thisMask!=0)
	{
	  nInfo |= 0x1
	    *(((thisMask&entry.suffix.ui)&(thisMask&nonIncorpFirstN))!=0);
	  nInfo |= 0x2
	    *(((thisMask&entry.suffix.ui)&(thisMask&nonIncorpSecondN))!=0);
          nInfo ^= 0x3*reverseFlag;
	} // ~if

	matchType_[oligoNum].errorType 
	  = (uchar)(numErrors|(nInfo<<2)|(0x80*reverseFlag));


      } // ~if	 
      else
      {
	// increment count if we are not already at capacity
	// if match is within 1 of an existing match, ignore it

      
	//	matchType_[oligoNum].r[numErrors]
	//	  +=( (matchType_[oligoNum].r[numErrors]!=0xFF)
	//	      &&( ((matchPosition_[oligoNum]-matchPos)
	//	   *(matchPosition_[oligoNum]-matchPos))
	//		  >1));
	// TC 31.10.03 
	// Above version can give an erroneous false on very rare occasions
	// (and so miss matches) if mP_[oN]-mP)^2 = -1,0,+1 mod 2^32
	// so instead use mp_[oN]-mP = -1,0,+1 iff mP_[oN]-mP+1 = 0,1,2
	// ... also avoids problem with unsigned quantities
	matchType_[oligoNum].r[numErrors]
	  +=( (matchType_[oligoNum].r[numErrors]!=0xFF)
	      && ( (matchPosition_[oligoNum]-matchPos+1) > 2 ));
      } // ~else

  //  cout << matchPosition_[oligoNum]-16777216 << " " << (uint) matchType_[oligoNum].errorType << " " << (uint) matchType_[oligoNum].r[0] << " "  << (uint) matchType_[oligoNum].r[1] << " " << (uint) matchType_[oligoNum].r[2] << endl;
#if (NUM_THREADS>1)
      pthread_mutex_unlock(&mutex_);
#endif
} // ~void MatchTable::addMatch( uint oligoNum, MatchPosition oligoPos )



// adjustMatchPos: make small corrections to the match position to correct
// for various quirks of the matching process (mainly to do with 
// incorporation errors and reverse complements)
void MatchTable::adjustMatchPos( const char* pOligo, 
		       const char dirChar,
		       char& firstN,
		       char& secondN,
		       MatchPosition& matchPos)
{
  int headSize, tailSize, numInternalNs;
  countNs( pOligo, headSize, tailSize, numInternalNs );

  // Print out '.' as the second N info character to signify there is only 
  // one internal N 
  if (numInternalNs<=1)
  {
    //    assert(secondN=='D');
    secondN='.';
  } // ~if

  if (numInternalNs==0)
  {
    assert(firstN=='D');
    firstN='.';
  } // ~if

  if (dirChar=='R')
  {
    // Need to shift match position forward by 1 for each non incorporation
    // (compensate for the extra Ns) 
    //   cout << matchPos << " -> "; 
    matchPos+=(firstN=='I');
    matchPos+=(secondN=='I');
    //  cout << matchPos << endl;
    // All Ns shoved at beginning which pushes match pos out - correct
    matchPos+=(headSize+tailSize);

    //    matchPos-=tailSize; // TC 27.1.6
  } // ~if
  else
  {
    assert(dirChar=='F');
    //    matchPos-=headSize; // TC 27.1.6
  } // ~else
} // ~MatchTable::adjustMatchPos








void MatchTableVerbose::print( OligoSource& oligos, 
			       const vector<string>& chromNames,
			       const vector<MatchPosition>& blockStarts,
			       const SuffixScoreTable& scoreTable,
			       int oligoLength)
{
  //  ErrorPositionMapper mapErrors(scoreTable, oligoLength);

  MatchPosition subtractTable[256];
  int chromTable[256];
  int thisChrom(0);
  uint j;
  uint errorPos1, errorPos2;

  //  bool isUniqueExact, isUniqueError;
  //  char buf[lineLength];
  const char* pOligo;
  ushort numErrors;
  char dirChar, firstN, secondN;
  Word nInfo;
  MatchPosition thisBlock, matchPos;

  for (vector<MatchPosition>::const_iterator i(blockStarts.begin());
       i!=blockStarts.end()-1;++i)
  {
    for ( j = *i; j != *(i+1); j+=blockSize)
    {
      subtractTable[j>>blockShift]=*i;
      chromTable[j>>blockShift]=thisChrom;
    } // ~for j
    ++thisChrom;
  } // ~for i

  oligos.rewind();

  //  for (int i(1); i <= numOligos_ ; i++ )
  for (uint i(1); i < matchPosition_.size() ; i++ )
  {

    if ( (pOligo=oligos.getNextOligo()) == NULL )
    {
      cerr << "OligoInfo: unexpectedly ran out of names!" << endl;
      exit (1);
    } // ~if

    fprintf( pOut_, "%d\t%s\t%s\t", i, oligos.getLastName(), pOligo );

    if (matchPosition_[i]>=blockRepeat)
    {
      if (matchPosition_[i]==qualityFailed)
	fprintf( pOut_, "QC");
      else if (matchPosition_[i]==repeatMasked)
	fprintf( pOut_,"RM");
      else
	fprintf( pOut_,"RB\t%u", matchPosition_[i]-blockRepeat);
    } // ~if
    else if (matchPosition_[i]==noMatch)
      fprintf( pOut_,"NM\t0\t0\t0");
    else 
    {
      numErrors=(matchType_[i].errorType&0x3);
      fprintf( pOut_,"%c%c\t%u\t%u\t%u",
	     ((matchType_[i].r[numErrors]>1)?'R':'U'),
	     '0'+numErrors,
	     ((numErrors==0)?((uint)matchType_[i].r[0]):0),
	     ((numErrors<=1)?((uint)matchType_[i].r[1]):0),
	     (uint)matchType_[i].r[2] );
      
      assert(matchType_[i].r[numErrors]!=0);
      if (matchType_[i].r[numErrors]==1)
      {
	// then we have a unique match - print its position

	// get direction
	dirChar = (((matchType_[i].errorType&0x80)!=0)?'R':'F');
	// get info on Ns
	nInfo=((Word)(matchType_[i].errorType&0x7C))>>2; // NNYYYYYN
	switch (nInfo)
	{
	case (noNonIncorps&0x3):
	  firstN='D';
	  secondN='D';
	  //	  fprintf( pOut_,"\tDD");
	  break;
	case (nonIncorpFirstN&0x3):
	  firstN='I';
	  secondN='D';
	  //	  fprintf( pOut_,"\tID");
	  break;
	case (nonIncorpSecondN&0x3):
	  firstN='D';
	  secondN='I';
	  //	  fprintf( pOut_,"\tDI");
	  break;
	case (nonIncorpBothNs&0x3):
	  firstN='I';
	  secondN='I';
	  //	  fprintf( pOut_,"\tII");
	  break;
	default:
	  assert(1==0);
	} // ~switch

	thisBlock = matchPosition_[i] >> blockShift;
	matchPos=matchPosition_[i]-subtractTable[thisBlock];

	adjustMatchPos( pOligo, dirChar, firstN, secondN, matchPos );

	fprintf
          ( pOut_,"\t%s\t%u\t%c\t%c%c", 
            chromNames[chromTable[thisBlock]].c_str(), 
            matchPos,//matchPosition_[i]-subtractTable[thisBlock],
            dirChar,
            firstN,
            secondN );

	if ((matchType_[i].errorType&0x3)==1)
	{
          errorPos1=(uint)(matchType_[i].r[0]&ucharErrorPosMask);
	  fprintf( pOut_,"\t%u", errorPos1);
          // fprintf( pOut_,"\t%u", (uint)matchType_[i].r[0]);
	} // ~if
	else if ((matchType_[i].errorType&0x3)==2)
	{
          errorPos1=(uint)(matchType_[i].r[0]&ucharErrorPosMask);
          errorPos2=(uint)(matchType_[i].r[1]&ucharErrorPosMask);
	  fprintf( pOut_,"\t%u\t%u", 
                   min(errorPos1, errorPos2), 
                   max(errorPos1, errorPos2) );
          //	  fprintf( pOut_,"\t%u\t%u", 
          // (uint)min(matchType_[i].r[0],matchType_[i].r[1]),
          // (uint)max(matchType_[i].r[0],matchType_[i].r[1]) );
	} // ~else if

      } // ~if
    } // ~else     
    fprintf( pOut_,"\n");
  } // ~for i
} // ~void MatchTableVerbose::print


// parseErrorInfo: parses the information stored in the error char
// - bottom 6 bits are the error position
// - using the top 2 bits and the original read it works 
//   out what the erroneous base should have been
void MatchTableAssembly::parseErrorInfo
( const char* oligoBuf, 
  const char errorInfo, 
  const char dirChar,
  uint& errorPos,
  uchar& shouldBe )
{
  errorPos=(uint)(errorInfo&ucharErrorPosMask);
  //  cout << endl << oligoBuf << endl;
  //  cout << errorPos << endl;
  // cout << (uint)whichBase[oligoBuf[errorPos-1]] << endl;;
  //  cout << (uint)((errorInfo&ucharErrorTypeMask)>>errorPosBits__) << endl; 
 

  //  uint shouldBeIndex
  //  ( whichBase[oligoBuf[errorPos-1]]
  //    ^((uint)((errorInfo&ucharErrorTypeMask)>>errorPosBits__)) );

  uint shouldBeIndex(((dirChar=='F')?(errorPos-1):(OLIGO_LENGTH-errorPos)));
  shouldBeIndex=whichBase[oligoBuf[shouldBeIndex]];
  shouldBeIndex^=((uint)((errorInfo&ucharErrorTypeMask)>>errorPosBits__));
  shouldBeIndex^=( ((uint)0x3)*(dirChar=='R') );

  //  cout << shouldBeIndex << endl;
  //  assert(shouldBeIndex<numDifferentBases);
  //  TBD Next line is a temporary fix. Should never occur - find out why
  if (shouldBeIndex>=numDifferentBases)
    shouldBeIndex=0;

  shouldBe=baseNames[shouldBeIndex];
} // ~MatchTableAssembly::parseErrorInfo



void MatchTableAssembly::print( OligoSource& oligos, 
				const vector<string>& chromNames,
				const vector<MatchPosition>& blockStarts,
				const SuffixScoreTable& scoreTable,
				int oligoLength)
{

  MatchPosition subtractTable[256];
  int chromTable[256];
  int thisChrom(0);
  uint j;
  uint errorPos1, errorPos2;
  uchar shouldBe1, shouldBe2;

  //  bool isUniqueExact, isUniqueError;
  //  char buf[lineLength];
  const char* pOligo;
  ushort numErrors;
  char dirChar, firstN, secondN;
  Word nInfo;
  MatchPosition thisBlock, matchPos;

  for (vector<MatchPosition>::const_iterator i(blockStarts.begin());
       i!=blockStarts.end()-1;++i)
  {
    for ( j = *i; j != *(i+1); j+=blockSize)
    {
      subtractTable[j>>blockShift]=*i;
      chromTable[j>>blockShift]=thisChrom;
    } // ~for j
    ++thisChrom;
  } // ~for i

  oligos.rewind();

  //  for (int i(1); i <= numOligos_ ; i++ )
  for (uint i(1); i < matchPosition_.size() ; i++ )
  {

    if ( (pOligo=oligos.getNextOligo()) == NULL )
    {
      cerr << "OligoInfo: unexpectedly ran out of names!" << endl;
      exit (1);
    } // ~if

    fprintf( pOut_, "%s\t%s\t", oligos.getLastName(), pOligo );

    if (matchPosition_[i]>=blockRepeat)
    {
      if (matchPosition_[i]==qualityFailed)
	fprintf( pOut_, "QC");
      else if (matchPosition_[i]==repeatMasked)
	fprintf( pOut_,"RM");
      else
	fprintf( pOut_,"RB\t%u", matchPosition_[i]-blockRepeat);
    } // ~if
    else if (matchPosition_[i]==noMatch)
      fprintf( pOut_,"NM\t0\t0\t0");
    else 
    {
      numErrors=(matchType_[i].errorType&0x3);
      fprintf( pOut_,"%c%c\t%u\t%u\t%u",
	     ((matchType_[i].r[numErrors]>1)?'R':'U'),
	     '0'+numErrors,
	     ((numErrors==0)?((uint)matchType_[i].r[0]):0),
	     ((numErrors<=1)?((uint)matchType_[i].r[1]):0),
	     (uint)matchType_[i].r[2] );
      
      assert(matchType_[i].r[numErrors]!=0);
      if (matchType_[i].r[numErrors]==1)
      {
	// then we have a unique match - print its position

	// get direction
	dirChar = (((matchType_[i].errorType&0x80)!=0)?'R':'F');
	// get info on Ns
	nInfo=((Word)(matchType_[i].errorType&0x7C))>>2; // NNYYYYYN
	switch (nInfo)
	{
	case (noNonIncorps&0x3):
	  firstN='D';
	  secondN='D';
	  //	  fprintf( pOut_,"\tDD");
	  break;
	case (nonIncorpFirstN&0x3):
	  firstN='I';
	  secondN='D';
	  //	  fprintf( pOut_,"\tID");
	  break;
	case (nonIncorpSecondN&0x3):
	  firstN='D';
	  secondN='I';
	  //	  fprintf( pOut_,"\tDI");
	  break;
	case (nonIncorpBothNs&0x3):
	  firstN='I';
	  secondN='I';
	  //	  fprintf( pOut_,"\tII");
	  break;
	default:
	  assert(1==0);
	} // ~switch

	thisBlock = matchPosition_[i] >> blockShift;
	matchPos=matchPosition_[i]-subtractTable[thisBlock];

	adjustMatchPos( pOligo, dirChar, firstN, secondN, matchPos );

	fprintf( pOut_,"\t%s\t%u\t%c\t%c%c", 
		 chromNames[chromTable[thisBlock]].c_str(), 
		 matchPos,//matchPosition_[i]-subtractTable[thisBlock],
		 dirChar,
		 firstN,
		 secondN
	       );

	if ((matchType_[i].errorType&0x3)==1)
	{
          parseErrorInfo
            ( pOligo, matchType_[i].r[0], dirChar, errorPos1, shouldBe1 );
          fprintf( pOut_, "\t%u%c", errorPos1, shouldBe1 );
          //	  fprintf( pOut_,"\t%u", (uint)matchType_[i].r[0]);
	} // ~if
	else if ((matchType_[i].errorType&0x3)==2)
	{
          parseErrorInfo
            ( pOligo, matchType_[i].r[0], dirChar, errorPos1, shouldBe1 );
          parseErrorInfo
            ( pOligo, matchType_[i].r[1], dirChar, errorPos2, shouldBe2 );
          //	  fprintf( pOut_,"\t%u\t%u", 
          //	 (uint)min(matchType_[i].r[0],matchType_[i].r[1]),
          //	 (uint)max(matchType_[i].r[0],matchType_[i].r[1]) );  
          if (errorPos1<errorPos2) 
          {
            fprintf( pOut_,"\t%u%c\t%u%c", 
                     errorPos1, shouldBe1, errorPos2, shouldBe2);
          } // ~if
          else 
          {
            fprintf( pOut_,"\t%u%c\t%u%c", 
                     errorPos2, shouldBe2, errorPos1, shouldBe1);
          } // ~else                  

	} // ~else if

      } // ~if
    } // ~else     
    fprintf( pOut_,"\n");
  } // ~for i

} // ~void MatchTableAssembly::print



// end of ELAND_outer.cpp
