/*
 * various disk IO and conversion routines for data analysis
 *
 * Time-stamp: <26 Jun 2006 17:00 BST IOProcess.cpp km>
 *
 * Copyright (c) Solexa 2003, 2004, 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "IOProcess.h"

long fread_uchar_array(const char filename[], unsigned char *data,
                       long nbytes, long offset)
{
    FILE *fp = fopen(filename, "r");
    if (fp) {
        if (offset > 0)
            fseek(fp, offset, SEEK_SET);
        long size = fread(data, sizeof(unsigned char), nbytes, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

long fwrite_uchar_array(const char filename[], char mode[],
                        unsigned char *data, long len)
{
    FILE *fp = fopen(filename, mode);
    if (fp) {
        long size = fwrite(data, sizeof(unsigned char), len, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

long fread_ushort_array(const char filename[], unsigned short *data,
                        long nelements, long offset)
{
    FILE *fp = fopen(filename, "r");
    if (fp) {
        if (offset > 0)
            fseek(fp, offset * sizeof(unsigned short), SEEK_SET);
        long size = fread(data, sizeof(unsigned short), nelements, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

long fwrite_ushort_array(const char filename[], char mode[],
                         unsigned short *data, long nelem)
{
    FILE *fp = fopen(filename, mode);
    if (fp) {
        long size = fwrite(data, sizeof(unsigned short), nelem, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

long fread_float_array(const char filename[], float *data, long len,
                       long offset)
{
    FILE *fp = fopen(filename, "r");
    if (fp) {
        if (offset > 0)
            fseek(fp, offset * sizeof(float), SEEK_SET);
        long size = fread(data, sizeof(float), len, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

long fwrite_float_array(const char filename[], char mode[], float *data,
                        long len)
{
    FILE *fp = fopen(filename, mode);
    if (fp) {
        long size = fwrite(data, sizeof(float), len, fp);
        fclose(fp);
        return size;
    }
    return 0;
}

/*
 * check if given file is readable
 */
static int file_exists(char filename[])
{
    FILE *fp = fopen(filename, "r");
    if (fp) {
        fclose(fp);
        return 1;
    }
    return 0;
}

/*
 * return the filesize of a given file
 */
long file_size(char filename[])
{
    FILE *fp = fopen(filename, "r");
    if (fp) {
        fclose(fp);
        struct stat stbuf;
        stat(filename, &stbuf);
        return (long)stbuf.st_size;
    }
    return -1;
}

#if ! defined(_WIN32)

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>

/*
 * create a new TCP/IP socket and connect to a server
 * at the given address and port
 */
int open_tcp_socket(char address[], int port)
{
    int sock;
    struct hostent *hostinfo;
    struct sockaddr_in serv_addr;

    // create socket
    sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (sock < 0) {
        perror("socket: Failed to create socket");
        exit(1);
    }

    // get host name
    hostinfo = gethostbyname(address);
    if (hostinfo == NULL) {
        fprintf(stderr, "Could not find host %s\n", address);
        exit(1);
    }

    // connect to server
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr = *((struct in_addr *)hostinfo->h_addr);
    serv_addr.sin_port = htons(port);
    if (connect(sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
        perror("connect: Failed to connect with server");
        exit(1);
    }
    return sock;
}

/*
 * send a data array through a socket
 */
void send_uchar_file(int sock, char filename[],
                     unsigned char data[], long datalen)
{
    char length[11];

    snprintf(length, 11, "%10d", strlen(filename));
    send(sock, length, strlen(length), 0);
    send(sock, filename, strlen(filename), 0);
    snprintf(length, 11, "%10ld", datalen);
    send(sock, length, strlen(length), 0);
    send(sock, data, datalen, 0);
}

#endif
