/*
 * Class to parse input from an image QC XML file.
 *
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <cstdlib>
#include <cstring>
#include "ImageXMLReader.h"

ImageXMLReader::ImageXMLReader()
{
    inImage = inDeblocked = inBase = inExposure = 0;
    index = 0;
}

ImageXMLReader::~ImageXMLReader()
{
}

void ImageXMLReader::startDocument()
{
    index = 0;
    int i;
    for (i = 0; i < 4; i++) {
        is_deblocked[i] = 0;
        exposure_times[i] = 0.;
        bases[i] = 'A';
    }
}

void ImageXMLReader::startElement(const char name[])
{
    if (!strcmp(name, "Image"))
        inImage = 1;
    if (!strcmp(name, "Base"))
        inBase = 1;
    if (!strcmp(name, "Deblocked"))
        inDeblocked = 1;
    if (!strcmp(name, "ExposureTime"))
        inExposure = 1;
}        

void ImageXMLReader::endElement(const char name[])
{
    if (!strcmp(name, "Image")) {
        inImage = 0;
        index ++;
    }
    if (!strcmp(name, "Base"))
        inBase = 0;
    if (!strcmp(name, "Deblocked"))
        inDeblocked = 0;
    if (!strcmp(name, "ExposureTime"))
        inExposure = 0;
}        

void ImageXMLReader::characters(const char ch[], int start, int length)
{
    if ((!inImage) || (index >= 4))
        return;
    if (inExposure)
        exposure_times[index] = atof(ch);
    if (inDeblocked)
        is_deblocked[index] = atoi(ch);
    if (inBase)
        bases[index] = ch[0];
}
