/*
 * Copyright (c) Solexa 2004, 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
//---------------------------------------------------------------------------

#ifndef ImagesH
#define ImagesH

#include <vector>
#include <string>
#include "Positions.h"
#include "Instrument.h"
#include "tifflib.h"

const int IMAGE_STATE_FILENAME     = (1<<0);
const int IMAGE_STATE_SIZE         = (1<<1);
const int IMAGE_STATE_PROPERTIES   = (1<<2);
const int IMAGE_STATE_IMAGE        = (1<<3);
const int IMAGE_STATE_DATA         = (1<<4);
const int IMAGE_STATE_OBJECTS      = (1<<5);

//---------------------------------------------------------------------------

class ImageProperties {
public:
    unsigned int   MinScale, MaxScale;
    float          StageXPosition, StageYPosition;
    char           StageLabel [80];
    char           ImageDescription[80];
    unsigned short BitsPerPixel;
    float          ExposureTime;
public:
    ImageProperties();
};

class Image {
private:
    int file_index;
    int nframes;
    char id[300];
    char filename[FILENAME_MAX];
    float threshold;
    float fwhm;
    float distance;
    int read_image_properties(TIFF_FILE *fp);
    st_instrument *instrument;
protected:
    unsigned short *image;
public:
    PositionData objects;
    int nx, ny;
    int state;
    int active;
    ImageProperties *properties;
    ImageData *data;

    Image();
    ~Image();
    void setFilename(const char fn[], int pos = 0, int npos = 0);
    void setSize(int x, int y, int totalpos = 0);
    int loadSize(TIFF_FILE *hd = 0);
    int loadProperties(TIFF_FILE *hd = 0);
    int load(TIFF_FILE *hd = 0);
    void setImage(unsigned short *pixels, int pnx, int pny);
    void clearImages();
    void clearData();
    void clear();
    void analyse();
    std::string get_id() const;
    std::string get_filename() const;
    void set_threshold(float val) { threshold = val; }
    float get_threshold() const { return threshold; }
    void set_fwhm(float val) { fwhm = val; }
    float get_fwhm() const { return fwhm; }
    void set_distance(float val) { distance = val; }
    float get_distance() const { return distance; }
    int get_frame_index();
    int get_frame_total();
    PositionData *get_objects();
    PositionData *get_objects_reference() { return &objects;}
    unsigned short *get_image();
    ImageData *getData();
};

class ImageList : public std::vector<Image *>
{
    std::vector <char *> filename;
    void clearImages();
public:
    int loadDimensions(const char filename[]);
    void addFile(const char *name) 
        {   char *fn = new char [FILENAME_MAX];
            snprintf(fn, FILENAME_MAX, name);
            filename.push_back(fn);
        }
    int load();
    void clearList();
    virtual ~ImageList();
    int createEmptyImages(int nx, int ny, int ni = 1);
    void write_info();
    void read_info();
};

#endif /* ImagesH */
