
/*
 *
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: sorting_partial.cpp,v 1.1 2006/10/24 13:15:15 lm Exp $
//



#include "sorting_partial.h"


//
// partial sorting for a matrix nxm (n=N, m=4)
//
// reorganize the data by sorting only the elements of one column and
// by shifting the rows;
// the sorting is done by using the 'Quicksort' algorithm developed by C.A.R. Hoare;
//

void partialSorting(const char sort_col, float *data, int left, int right)
{
    int col, i, j, r, m;
    float mean, temp_1c, temp_2c, temp_3c, temp_4c;

    i = left;
    j = right;

    if (sort_col == '1')
        col = 0;
    else if (sort_col == '2')
        col = 1;
    else if (sort_col == '3')
        col = 2;
    else if (sort_col == '4')
        col = 3;

    r = 0;
    m = ((col + i * NO_COLS) + (col + j * NO_COLS)) / 2;
    for (int k = 0;; k++) {
        r = (col + k * NO_COLS);
        if (m == r)
            break;
        if (m < r) {
            k--;
            r = (col + k * NO_COLS);
            m = r;
            break;
        }
    }
    mean = data[m];

    do {
        while (data[col + i * NO_COLS] < mean && i < right)
            i++;
        while (mean < data[col + j * NO_COLS] && j > left)
            j--;
        if (i <= j) {
            temp_1c = data[i * NO_COLS];
            data[i * NO_COLS] = data[j * NO_COLS];
            data[j * NO_COLS] = temp_1c;
            temp_2c = data[1 + i * NO_COLS];
            data[1 + i * NO_COLS] = data[1 + j * NO_COLS];
            data[1 + j * NO_COLS] = temp_2c;
            temp_3c = data[2 + i * NO_COLS];
            data[2 + i * NO_COLS] = data[2 + j * NO_COLS];
            data[2 + j * NO_COLS] = temp_3c;
            temp_4c = data[3 + i * NO_COLS];
            data[3 + i * NO_COLS] = data[3 + j * NO_COLS];
            data[3 + j * NO_COLS] = temp_4c;
            i++;
            j--;
        }
    } while (i <= j);

    if (left < j)
        partialSorting(sort_col, data, left, j);
    if (i < right)
        partialSorting(sort_col, data, i, right);


}
