
/*
 *
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: estimate_xtalk_mx.cpp,v 1.3 2006/11/22 09:30:12 lm Exp $
//



#include "estimate_xtalk_mx.h"
#include "optimise_xtalk_mx.h"
#include "savinio_common.h"
#include "scale_normalise_xtalk_mx.h"


//
// (new data) = (cross-talk matrix)^-1 * (data)
//
void applyXTalkMx(const int no_i_values, float *i_data, Matrix & mx,
                  float *i_data_new)
{
    Matrix line(4, 1);
    Matrix line_new(4, 1);
    Matrix mx_inv;

    mx_inv = mx.inverse();

    for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
        for (int c = 0; c < NO_BASES; ++c) {
            line.data[c] = i_data[c + r * NO_BASES];
        }
        line_new = mx_inv * line;
        for (int c = 0; c < NO_BASES; ++c) {
            i_data_new[c + r * NO_BASES] = line_new.data[c];
        }
    }

}


void estimateXTalkMx(const int no_i_values, float *i_data,
                     Matrix & mx_normalised)
{
    // intensities (temporary)
    float *i_data_1 = new float[no_i_values];

    // cross-talk matrix (temporary)
    Matrix mx1, mx2;
    // cross-talk matrix (unscaled)
    Matrix mx_unscaled;


    // initial estimate of the unscaled cross-talk matrix
    mx1.identity_matrix();

    applyXTalkMx(no_i_values, i_data, mx1, i_data_1);

    optimiseXTalkMx(no_i_values, i_data_1, mx2);

    // final estimate of the unscaled cross-talk matrix
    mx_unscaled = mx1 * mx2;

    // scaling and normalising the cross-talk matrix  
    scale_normaliseXTalkMx(no_i_values, i_data, mx_unscaled, mx_normalised);


    delete [] i_data_1;

}
