
/*
 *
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: fitting.cpp,v 1.3 2006/11/22 09:30:55 lm Exp $
//



#include "fitting.h"
#include "lav_estimation.h"
#include "Optimise.h"
#include "savinio_common.h"


//
// data fitting
//
// fitting the data to a straight line through the origin  y = b*x ;
// fitting the data by using the Least Absolute Values method (L1 norm method, L1 regression);
//
void fitting_LAV(float *xaxis_data, float *yaxis_data, float &slope)
{

    int i;
    int funcallcount;
    float ax, bx, cx;
    float fa, fb, fc;
    float xtol;
    float xmin;
    float fmin;
    float xbracket[3];


    // structure which contains the arguments for
    // the function associated to the LAV
    struct args_lav s_args_lav;

    s_args_lav.no_el = NO_BINS;
    s_args_lav.data_x = new float[NO_BINS];
    for (i = 0; i < NO_BINS; ++i) {
        s_args_lav.data_x[i] = xaxis_data[i];
    }
    s_args_lav.data_y = new float[NO_BINS];
    for (i = 0; i < NO_BINS; ++i) {
        s_args_lav.data_y[i] = yaxis_data[i];
    }

    struct args_lav *p_args_lav;

    p_args_lav = &s_args_lav;

    void *p_args_lav_v = (void *)p_args_lav;

    ax = (float)1 / 2.0;
    bx = 2.0;

    // 'LAVFunc(x, args)'
    //  x = slope of the fitting line (independent variable)

    // bracketing procedure
    bracket(LAVFunc, p_args_lav_v, &ax, &bx, &cx, &fa, &fb, &fc);

    xbracket[0] = ax;
    xbracket[1] = bx;
    xbracket[2] = cx;
    xtol = 2.0e-3;

    // minimising of the function associated to the LAV by using the Brent's method
    brent(LAVFunc, p_args_lav_v, xbracket, xtol, &xmin, &fmin, &funcallcount);

    slope = xmin;


    delete [] s_args_lav.data_x;
    delete [] s_args_lav.data_y;

}
