
/*
 *
 * Copyright (c) 2006-2007 Solexa
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: input_process.cpp,v 1.5 2007/02/28 16:57:27 lm Exp $
//



#include <string>
#include <vector>

#include "input_process.h"
#include "savinio_common.h"

#include "savinio_debugging.h"


InputProcess::InputProcess(const char *input_file)
:fs_in(input_file)
{

}


//
// input file:       intensity file (Phoenix intensity output file)
// input file name: 's_no-lane_no-tile_no-cycle_int.txt'
//                   (s=sample, int=intensities)
//
// the intensity file consists of four intensity values per cluster (corresponding
// to the uncorrected A, C, G, T channels in this order), one line per cluster;
// it contains the intensities associated with all clusters, for a given cycle,
// for a given tile, for a given lane;
//
void InputProcess::fromInput_Intensities(float *intensities, int &no_i_values)
{
    // input processing

    float i_value;
    float* i_cluster = new float[NO_BASES];
    std::string line;

    // reading lines from input file
    // (each line is associated with a cluster and it contains
    //  four intensity values)
    while(getline(fs_in, line)){
  
        int index_bases = 0;
        char* start;
        char* end;
        const char* end_t;
    
        end_t = line.c_str();
        end = const_cast<char*>(end_t);
        do {
            start = end;
            i_value = (float)strtod(start, &end);
            i_cluster[index_bases] = i_value;
            index_bases++;
        } while ((index_bases < NO_BASES) && (end != NULL) && (end != start));

        // storing intensity values only from `effective` clusters 
        if ((i_cluster[0] != 0) || (i_cluster[1] != 0) ||
            (i_cluster[2] != 0) || (i_cluster[3] != 0)) {
            for (int i = 0; i<NO_BASES; i++) {
                intensities[no_i_values] = i_cluster[i];
                no_i_values++;
            }
        }

        // reading up to a maximum no. of lines (clusters)
        if (no_i_values == (MAX_NO_CLUSTERS * NO_BASES)) break;

    }


    delete [] i_cluster;


    //--
    #ifdef DEBUG_IO
        std::cout << "\n>> debug - I/O processing" << std::endl;

        std::cout << "input file, no. of intensity values: " << no_i_values << std::endl;

    #endif // DEBUG_IO
    //--


}
