
/*
 *
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: optimise_xtalk_mx.cpp,v 1.3 2006/11/22 09:31:31 lm Exp $
//



#include "fitting.h"
#include "optimise_xtalk_mx.h"
#include "sampling.h"
#include "savinio_common.h"

#include "savinio_debugging.h"


void optimiseXTalkMx(const int no_i_values, float *i_data, Matrix & mx)
{
    // selected data (after sampling) for the X and Y axes
    float *xaxis_data = new float[NO_BINS];
    float *yaxis_data1 = new float[NO_BINS];
    float *yaxis_data2 = new float[NO_BINS];
    float *yaxis_data3 = new float[NO_BINS];

    // off-diagonal elements of the cross-talk matrix
    float *mx_off_diagonals = new float[NO_OFF_DIAGONALS];

    // bases
    std::string bases("ACGT");

    int j;


    for (j = 0; j < NO_OFF_DIAGONALS; ++j) {
        mx_off_diagonals[j] = 0.0;
    }


    // first estimate of the cross-talk matrix:
    // . sampling
    // . fitting of the selected data and estimate of the slopes
    //   of the 12 archetypal directions
    for (int i = 0; i < NO_BASES; ++i) {
        sampling1(bases.at(i), no_i_values, i_data, xaxis_data, yaxis_data1,
                  yaxis_data2, yaxis_data3);
        fitting_LAV(xaxis_data, yaxis_data1,
                    mx_off_diagonals[i * (NO_OFF_DIAGONALS / NO_BASES)]);
        fitting_LAV(xaxis_data, yaxis_data2,
                    mx_off_diagonals[1 + i * (NO_OFF_DIAGONALS / NO_BASES)]);
        fitting_LAV(xaxis_data, yaxis_data3,
                    mx_off_diagonals[2 + i * (NO_OFF_DIAGONALS / NO_BASES)]);
    }

    j = 0;
    for (int c = 0; c < mx.cols; ++c) {
        for (int r = 0; r < mx.rows; ++r) {
            if (r == c) {
                mx.data[c + r * mx.cols] = 1;
            }
            else {
                mx.data[c + r * mx.cols] = mx_off_diagonals[j];
                j++;
            }
        }
    }


    delete [] xaxis_data;
    delete [] yaxis_data1;
    delete [] yaxis_data2;
    delete [] yaxis_data3;
    delete [] mx_off_diagonals;


    //--
    #ifdef DEBUG_XTALK_MATRIX_1
        std::cout << "\n>> debug - unscaled cross-talk matrix, first estimate" << std::endl;

        mx.display();

    #endif // DEBUG_XTALK_MATRIX_1
    //--


}
