
/*
 *
 * Copyright (c) 2006-2007 Solexa
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: sampling.cpp,v 1.7 2007/03/01 11:32:14 lm Exp $
//



#include "sampling.h"
#include "savinio_common.h"
#include "sequential_search.h"
#include "sorting_standard.h"
#include "Sorting.h"

#include "savinio_debugging.h"


float getYAxisData(const float *bin_data, const int b_start, const int b_end)
{
    float *bin_data_t = new float[(b_end - b_start)];

    int i = 0;
    for (int j = b_start; j < b_end; ++j) {
        bin_data_t[i] = bin_data[j];
        i++;
    }

    standardSorting(bin_data_t, 0, ((b_end - b_start) - 1));

    int index_lbound = (int)(PERC_LOWER_Y_S1 * (b_end - b_start));

    float yaxis_data_el = bin_data_t[index_lbound];

    delete [] bin_data_t;

    return yaxis_data_el;

}


//
// 'data sampling 1'
//
// sampling the intensities associated with the X and Y axes
//
// (note that there is one X-axis for each of the 12 archetypal directions and
// there are four different X-axis, one for each color)
//
void sampling1(const char base, const int no_i_values, const float *i_data,
               float *xaxis_data, float *yaxis_data1, float *yaxis_data2,
               float *yaxis_data3)
{

    // intensities (temporary)
    float *i_data_t = new float[no_i_values];

    // intensities associated with one base (A, C, G, T)
    float *i_baseX = new float[(no_i_values / NO_BASES)];

    // intensities stored in the bins
    float *bin_data1 = new float[no_i_values];
    float *bin_data2 = new float[no_i_values];
    float *bin_data3 = new float[no_i_values];

    // element position list; original position for each sorted element
    long *indeces = new long[(no_i_values / NO_BASES)];

    // bin occupancy list; occupancy for each bin
    int *bin_occupancy = new int[NO_BINS];

    float Xel_lbound;
    float Xel_ubound;


    //--
    // sampling the X-axis

    if (base == 'A') {
        for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
            i_baseX[r] = i_data[r * NO_BASES];
        }
    }
    if (base == 'C') {
        for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
            i_baseX[r] = i_data[1 + r * NO_BASES];
        }
    }
    if (base == 'G') {
        for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
            i_baseX[r] = i_data[2 + r * NO_BASES];
        }
    }
    if (base == 'T') {
        for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
            i_baseX[r] = i_data[3 + r * NO_BASES];
        }
    }

    // sorting
    for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
        indeces[r] = r;
    }
    quicksort_index(i_baseX, indeces, long ((no_i_values / NO_BASES)));

    // binning
    int index_lbound = (int)(PERC_LOWER_X_S1 * (no_i_values / NO_BASES));
    int index_ubound = (int)(PERC_UPPER_X_S1 * (no_i_values / NO_BASES));

    Xel_lbound = i_baseX[index_lbound];
    Xel_ubound = i_baseX[index_ubound];

    for (int b = 0; b < NO_BINS; b++) {
        xaxis_data[b] = Xel_lbound + (b + 0.5) * (Xel_ubound - Xel_lbound) / NO_BINS;
    }


    //--
    // sampling the Y-axis (three Y-axes)

    for (int i = 0; i < no_i_values; ++i) {
        i_data_t[i] = i_data[i];
    }

    // binning
    int r = 0;
    int i = 0;
    int j = 0;
    float Xel_max;

    while (i_baseX[r] < Xel_lbound) {
        r++;
    }
    if (base == 'A') {
        for (int b = 0; b < NO_BINS; b++) {
            Xel_max = Xel_lbound + (b + 1) * (Xel_ubound - Xel_lbound) / NO_BINS;
            while (i_baseX[r] < Xel_max) {
                // bin_data1 for base 'C'
                bin_data1[i] = i_data_t[1 + indeces[r] * NO_BASES];
                // bin_data2 for base 'G'
                bin_data2[i] = i_data_t[2 + indeces[r] * NO_BASES];
                // bin_data3 for base 'T'
                bin_data3[i] = i_data_t[3 + indeces[r] * NO_BASES];
                r++;
                i++;
                j++;
            }
            bin_occupancy[b] = j;
            j = 0;
            Xel_max = 0;
        }
    }
    if (base == 'C') {
        for (int b = 0; b < NO_BINS; b++) {
            Xel_max = Xel_lbound + (b + 1) * (Xel_ubound - Xel_lbound) / NO_BINS;
            while (i_baseX[r] < Xel_max) {
                // bin_data1 for base 'A'
                bin_data1[i] = i_data_t[indeces[r] * NO_BASES];
                // bin_data2 for base 'G'
                bin_data2[i] = i_data_t[2 + indeces[r] * NO_BASES];
                // bin_data3 for base 'T'
                bin_data3[i] = i_data_t[3 + indeces[r] * NO_BASES];
                r++;
                i++;
                j++;
            }
            bin_occupancy[b] = j;
            j = 0;
            Xel_max = 0;
        }
    }
    if (base == 'G') {
        for (int b = 0; b < NO_BINS; b++) {
            Xel_max = Xel_lbound + (b + 1) * (Xel_ubound - Xel_lbound) / NO_BINS;
            while (i_baseX[r] < Xel_max) {
                // bin_data1 for base 'A'
                bin_data1[i] = i_data_t[indeces[r] * NO_BASES];
                // bin_data2 for base 'C'
                bin_data2[i] = i_data_t[1 + indeces[r] * NO_BASES];
                // bin_data3 for base 'T'
                bin_data3[i] = i_data_t[3 + indeces[r] * NO_BASES];
                r++;
                i++;
                j++;
            }
            bin_occupancy[b] = j;
            j = 0;
            Xel_max = 0;
        }
    }
    if (base == 'T') {
        for (int b = 0; b < NO_BINS; b++) {
            Xel_max = Xel_lbound + (b + 1) * (Xel_ubound - Xel_lbound) / NO_BINS;
            while (i_baseX[r] < Xel_max) {
                // bin_data1 for base 'A'
                bin_data1[i] = i_data_t[indeces[r] * NO_BASES];
                // bin_data2 for base 'C'
                bin_data2[i] = i_data_t[1 + indeces[r] * NO_BASES];
                // bin_data3 for base 'G'
                bin_data3[i] = i_data_t[2 + indeces[r] * NO_BASES];
                r++;
                i++;
                j++;
            }
            bin_occupancy[b] = j;
            j = 0;
            Xel_max = 0;
        }
    }

    int b_start = 0;
    int b_end = 0;

    for (int b = 0; b < NO_BINS; b++) {
        b_end = b_start + bin_occupancy[b];
        yaxis_data1[b] = getYAxisData(bin_data1, b_start, b_end);
        yaxis_data2[b] = getYAxisData(bin_data2, b_start, b_end);
        yaxis_data3[b] = getYAxisData(bin_data3, b_start, b_end);
        b_start = b_end;
    }

 
    //--

    delete [] i_data_t;
    delete [] i_baseX;
    delete [] bin_data1;
    delete [] bin_data2;
    delete [] bin_data3;
    delete [] indeces;
    delete [] bin_occupancy;


    //--
    #ifdef DEBUG_SAMPLING1
    std::cout << "\n>> debug - sampling1";

    std::cout << "\nX axis: " << std::endl;
    for (int i = 0; i < NO_BINS; ++i) {
        std::cout << xaxis_data[i] << " ";
    }
    std::cout << "\nY axis 1: " << std::endl;
    for (int i = 0; i < NO_BINS; ++i) {
        std::cout << yaxis_data1[i] << " ";
    }
    std::cout << "\nY axis 2: " << std::endl;
    for (int i = 0; i < NO_BINS; ++i) {
        std::cout << yaxis_data2[i] << " ";
    }
    std::cout << "\nY axis 3: " << std::endl;
    for (int i = 0; i < NO_BINS; ++i) {
        std::cout << yaxis_data3[i] << " ";
    }
    std::cout << "\n";

    #endif // DEBUG_SAMPLING1
    //--


}


//
// 'data sampling 2'
//
// sampling the intensities
//
// . choose the brightest intensities for each clusters
//
void sampling2(const int no_i_values, const float *i_data,
               float *i_data_bA, float *i_data_bC, float *i_data_bG,
               float *i_data_bT, int &no_i_values_bA, int &no_i_values_bC,
               int &no_i_values_bG, int &no_i_values_bT)
{
    int index_el_max;
    int ia, ic, ig, it;
    float el_max;

    // intensities associated with a cluster
    float *i_cluster = new float[NO_BASES];
    float *i_cluster_sorted = new float[NO_BASES];

    ia = 0;
    ic = 0;
    ig = 0;
    it = 0;
    for (int r = 0; r < (no_i_values / NO_BASES); ++r) {
        for (int c = 0; c < NO_BASES; ++c) {
            i_cluster[c] = i_data[c + r * NO_BASES];
        }
        for (int c = 0; c < NO_BASES; ++c) {
            i_cluster_sorted[c] = i_data[c + r * NO_BASES];
        }

        standardSorting(i_cluster_sorted, 0, (NO_BASES - 1));
        el_max = i_cluster_sorted[(NO_BASES - 1)];
        index_el_max = sequentialSearch(i_cluster, NO_BASES, el_max);

        if (index_el_max == 0) {
            i_data_bA[ia] = el_max;
            ia++;
        }
        else if (index_el_max == 1) {
            i_data_bC[ic] = el_max;
            ic++;
        }
        else if (index_el_max == 2) {
            i_data_bG[ig] = el_max;
            ig++;
        }
        else if (index_el_max == 3) {
            i_data_bT[it] = el_max;
            it++;
        }

    }

    no_i_values_bA = ia;
    no_i_values_bC = ic;
    no_i_values_bG = ig;
    no_i_values_bT = it;

    delete [] i_cluster;
    delete [] i_cluster_sorted;


}


//
// 'data sampling 3'
//
// sampling the intensities
//
// . filter out the very low intensities
//
void sampling3(const float *i_data_i, const int no_i_values_i,
               float *i_data_f, int& no_i_values_f)
{

    int index_ubound;
    float lbound;

    index_ubound = (int)(PERC_UPPER_X_S3 * (no_i_values_i + 1));
  
    lbound = (PERC_AGE_S3 * i_data_i[(index_ubound-1)]);

    int j = 0;
    for (int i = 0; i<no_i_values_i; i++) {
        if (i_data_i[i] > lbound) {
            i_data_f[j] = i_data_i[i];
            j++;
        }
    }

    no_i_values_f = j;


}
