
/*
 *
 * Copyright (c) 2006-2007 Solexa
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: savinio.cpp,v 1.13 2007/02/28 16:51:09 lm Exp $
//



//
// program name: 'Savinio'
// version:       0.7.0
//             
// A program to estimate the cross-talk matrix for the 'color-correction'
// in four-dye florescence-based DNA sequencing.
//



#include <iostream>
#include <fstream>

#include "display_xtalk_mx.h"
#include "estimate_xtalk_mx.h"
#include "savinio_common.h"

#include "input_process.h"
#include "output_process.h"

#include "savinio_debugging.h"
#include "savinio_exceptions.h"


static void usage();


int main(int argC, char *argV[])
{

    //--  

    char *input_file = NULL;
    char *output_file = NULL;
    bool output = false;

    if (argC < 2) {
        usage();
        return (1);
    }

    if (!strcmp(argV[1], "-h")) {
        usage();
        return (2);
    }
    else if (!strcmp(argV[1], "-o")) {
        output = true;
        output_file = argV[2];
        input_file = argV[3];
    }
    else {
        input_file = argV[1];
    }


    //--

    int no_i_values = 0;

    // intensities
    float *i_data = new float[NO_BASES * MAX_NO_CLUSTERS];

    // cross-talk matrix    
    Matrix mx;


    InputProcess in(input_file);
    OutputProcess out(output_file);

    // reading input file   
    in.fromInput_Intensities(i_data, no_i_values);

    try {
        // input validation
        if (no_i_values < (MIN_NO_CLUSTERS * NO_BASES)) {
            Exception01 e1(input_file, no_i_values);
            throw e1;
        }

        // estimating the cross-talk matrix  
        estimateXTalkMx(no_i_values, i_data, mx);

    }
    catch(Exception01 e1) {
        e1.display();

        // initial estimate of the cross-talk matrix
        mx.identity_matrix();

    }


    // printing out the cross-talk matrix
    displayXTalkMx(mx);

    // writing output file
    if (output == true) {
        out.toOutput_Matrix(mx);
    }

    delete [] i_data;


    return (0);

}


static void usage()
{
    std::cout << "\n---------------------------------------------------------------\n"
                 "                     ** Savinio **\n"
                 "                vers. 0.7.0 - Solexa Ltd.\n\n" " Usage:\n"
                 "    $ savinio [option] <input file> \n\n" " Options:\n"
                 "    -h                help\n"
                 "    -o <output file>  writing to output file\n\n"
              << std::endl;

}
