/*
 * Copyright (c) Solexa 2004, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#ifndef _POSITIONS_H_
#define _POSITIONS_H_
//---------------------------------------------------------------------------
#include <cstdio>
#include "ImageProcess.h"

int write_positions(FILE *filehandle, st_fnoisepixel *positions, int n);

// binary operations
void or_positions(st_fposition *positions, int *count,
                  st_fposition *positions1, int count1,
                  st_fposition *positions2, int count2,
                  int nx, int ny, float min_dist);
void subtract_positions(st_fposition *positions, int *count,
                        st_fposition *positions1, int count1,
                        st_fposition *positions2, int count2,
                        int nx, int ny, float min_dist);
void and_positions(st_fposition *positions, int *count,
                   st_fposition *positions1, int count1,
                   st_fposition *positions2, int count2,
                   int nx, int ny, float min_dist);


class Positions
{
public:
    st_fposition *positions;
    int count;
    float shiftX, shiftY;
    int nx, ny;
private:
    void initialise();
    void resize(int num);
public:
    Positions();
    Positions(const Positions& temp);
    Positions(int n = 0, int x = N_PIX, int y = N_PIX);
    ~Positions();
    int write(const char filename[]);
    int read(const char filename[]);
    void shift(float x, float y);
    int getSize() const { return count; };
    st_fposition *getPositions() const { return positions; }
    void align(const Positions& reference, int maxlag,
               float fwhm);
    Positions *bin_or(const Positions &s2, float dist);
    Positions *bin_subtract(const Positions &s2, float dist);
    Positions *bin_and(const Positions &s2, float dist);
};

class PositionData
{
private:
    st_fnoisepixel *data;
    int count;
public:
    float shiftX, shiftY;
    int nx, ny;
public:
    PositionData(int x = 0, int y = 0) :
           data(0), count(0), shiftX(0.), shiftY(0.) { nx = x; ny = y; };
    ~PositionData()
        {
            if (count > 0)
                delete [] data;
        };
    void setData(st_fnoisepixel *dt, int n)
        {
            data = dt;
            count = n;
        }
    int getSize() const { return count; }
    st_fnoisepixel *getData() const { return data; }
    void shift(float x, float y);
    Positions *getPositionsCopy() const
        {
            Positions *pos = new Positions(count);
            int i;
            for (i = 0; i < count; i++) {
               pos->positions[i].x = data[i].x;
               pos->positions[i].y = data[i].y;
            }
            return pos;
        }
    st_pixel *getPixelsCopy() const;
    int write(const char filename[]);
    int read(const char filename[]);
    void align(const PositionData& ref, int maxlag, float fwhm,
               int nx, int ny);
};
#endif /* _POSITIONS_H_ */
