/*
 * Copyright (c) Solexa 2004, 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#ifndef ProcessH
#define ProcessH
//---------------------------------------------------------------------------

#include <string>
#include <vector>
#include "ImageProcess.h"
#include "Images.h"
#include "QCRemapping.h"
#include "ProcessAlign.h"
#include "QCWriter.h"

const char software[] = "Firecrest";
const char software_version[] = "1.8.28";

//---------------------------------------------------------------------------

class DataProcessor
{
  private:
    OffsetTracker *offset_tracker;
    ProcessAlign *aligner;
    int   imagemax, colourmax;
    int   max_object_number;

    AlignmentTransform *default_transforms;
    float max_stage_offset, max_colour_offset;
    unsigned char *bases;
    float fwhm, dist;
    int   nx, ny;
    void  clear_memory();
  public:
    DataProcessor();
    ~DataProcessor();
    void initialise();
#if 0
    void processCamera(unsigned char *cameradata,
                       int tile, int colour, int cycle);
#endif
    void analyseImage(Image *image, 
                      st_fposition *referencepositions, int referencecount,
                      int tile, int colour, int cycle);
    int  validateOffsets(std::vector<Image * > images, int tile, int cycle);
    void buildReference(std::vector<Image * >images,
                        st_fposition *oldreference, int oldcount,
                        st_fposition **newreference, int *newcount);
    void analyseTile(std::vector<Image * > images,
                     st_fposition *referencepositions, int referencecount,
                     float *intensities, float *noise, int *refindices = 0);
    void analyseTile(std::vector<Image * >images,
                     int tile, int cycle, int *flag,
                     st_fposition *referencepositions, int referencecount,
                     float *intensities, float *noise, int *refindices = 0);
    void setParameters(int colourmax, int x, int y,
                       float fwhm = 2.2, float dist = 1.1);
    void setDefaultTransforms(AlignmentTransform *transforms, int nc);
    void setMaxAlignmentOffsets(float stage_offset, float colour_offset = 2.)
        { max_stage_offset = stage_offset; 
          max_colour_offset = colour_offset; }
    void displayOffsets();
    void saveOffsets();
};

#endif /* ProcessH */
