/*
 * Time-stamp: <26 Jun 2006 17:04 BST ProcessAlign.h km>
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

//---------------------------------------------------------------------------

#ifndef _PROCESSALIGN_H
#define _PROCESSALIGN_H

#include "ImageUtil.h"

struct AlignmentRecord {
public:
    int  nx, ny, firstx, firsty;
    float offsetx, offsety;
    float correlation_maximum, correlation_minimum, correlation_width;
    virtual ~AlignmentRecord() {};
};
int validateAlignment(AlignmentRecord *transforms, int ntrans,
                      float max_offset = 2., int includeflag = -1);

void align_images(float *image1, float *image2,
                  int nx, int ny, int xmaxlag, int ymaxlag,
                  AlignmentRecord *alignment);
void align_images_fft(float *image1, float *image2,
                      int nx, int ny, int xmaxlag, int ymaxlag,
                      AlignmentRecord *alignment);
void align_fft_images(float *image1, float *image2, int nx, int ny, 
                      int xmaxlag, int ymaxlag,
                      AlignmentRecord *alignment);
void create_clean_fft_image(float *fourier, int nx, int ny,
                            st_pixel *positions, int npos,
                            float fwhm, int uniform);
void align_positions(st_pixel *positions1, int n1,
                     st_pixel *positions2, int n2,
                     int nx, int ny, float fwhm, int xmaxlag, int ymaxlag,
                     AlignmentRecord *alignment);
int calculate_object_overlap(st_pixel *positions1, int n1,
                             st_pixel *positions2, int n2,
                             int nx, int ny, int xoff, int yoff,
                             float min_dist);
int calculate_object_overlap(st_fnoisepixel *positions1, int n1,
                             st_fnoisepixel *positions2, int n2,
                             int nx, int ny, int xoff, int yoff,
                             float min_dist);

class AlignmentTransform : public AlignmentRecord
{
 public:
    float scalex, scaley;
    AlignmentRecord *local_alignment;
    int nlocal;
    AlignmentTransform(int x = 0, int y = 0)
        { offsetx = 0.f; offsety = 0.f;
          scalex = 0.f; scaley = 0.f;
          nx = x; ny = y;
          correlation_minimum = 0.f; correlation_maximum = 0.f;
          correlation_width = 0.f;
          local_alignment = 0; nlocal = 0; }
    AlignmentTransform(const AlignmentTransform &ref)
    {
        AlignmentTransform();
        copy(ref);
    }
    ~AlignmentTransform()
    {
        if (nlocal > 0)
            delete [] local_alignment;
    }
    void copy(const AlignmentTransform& ref);
    void transformForward(float *x, float *y) const
    {
        *x += (*x - nx/2) * scalex + offsetx;
        *y += (*y - ny/2) * scaley + offsety;
    }
    void transformReverse(float *x, float *y) const
    {
        *x = (*x + nx/2.f*scalex - offsetx) / (1.f+scalex);
        *y = (*y + ny/2.f*scaley - offsety) / (1.f+scaley);
    }
    void transformReverse(st_fnoisepixel *pixel, int n) const;
};

int loadAlignmentTransforms(const char filename[],
                            AlignmentTransform *transforms, int nmax);

class ProcessAlign
{
    float *reference_fourier_image;
    st_pixel *reference_positions;
    int reference_count;
    float fwhm;
    int maxlagx, maxlagy;
    int nx, ny, fnx, fny;
    int numregionx, numregiony;
    int regionsizex, regionsizey;
 public:
    ProcessAlign(st_fposition *refpositions, int refcount,
                 int nx2, int ny2, float fwhm);
    ~ProcessAlign();
    void setMaximumOffsets(int xlag, int ylag = -1);
    void alignPixelPositions(st_fnoisepixel *positions, int npos,
                             AlignmentRecord *offsets, float fwhm = -1.f);
    void setRegionParameters(int numx, int numy, int sizex = -1, int sizey = -1); 
    void alignRegionPixelPositions(st_fnoisepixel *positions, int npos,
                                   AlignmentTransform *transform);
};

#endif /* _PROCESSALIGN_H */
