/*
 * Copyright (c) Solexa 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
#ifndef PROCESSREMAPPING_H_
#define PROCESSREMAPPING_H_

#include "ImageUtil.h"
#include "ProcessAlign.h"

class RemappingLookup {
    int *lookup;
    int nx, ny;
    st_fposition *reference;
    st_fposition *newpositions;
    st_fnoisepixel *candidate_newpositions;
    int *refindex;
    int nref;
    int nnew, nmax, ncandnew;
    void remap(st_fnoisepixel *positions, int *new_index, int npos,
               int *ref_index, float min_dist);
    void remapPosition(st_fnoisepixel position,
                       int *new_index, float *new_nearest_dist, 
                       float min_dist, st_fposition *reference, int nref,
                       int pos, int *ref_index, float *ref_nearest_distance);
public:
    RemappingLookup(int x, int y);
    virtual ~RemappingLookup();
    void setReference(st_fposition *positions, int nref);
    void clearReference();
    void clearNewPositions();
    void setMaxNewPositions(int n);
    void getNewPositions(st_fposition *positions, int nmax);
    void addNewPositions(st_fnoisepixel *positions, int npos, float min_dist);
    void findNewPositions(float min_dist);
    void addNewPositionsToReference();
    void remap(st_fnoisepixel *positions, int npos, float min_dist);
    int getNumberNewPositions() { return nnew; }
    st_fposition *getNewPositions() { return newpositions; }
    int *getIndex() { return refindex;}
    
};


void remap_colour(st_fposition *refpositions, int *refindices, int nref,
                  st_fnoisepixel *data,  int ndata,
                  float *intensities, float *sigma, int ncol, int icol);
void complete_intensities(st_fposition *refpositions, int *refindices, 
                          int nref, int ncol, int icol, 
                          float *image, float *noise_image, int nx, int ny,
                          float *intensities, float *noise_dispersions,
                          float fwhm, float min_dist,
                          AlignmentTransform *transform);

#endif /*PROCESSREMAPPING_H_*/
