/*
 * Class to write image analysis QC output to XML
 *
 * (c) Copyright 2006 Solexa
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
#include "QCWriter.h"

QCWriter::QCWriter()
{
}

QCWriter::~QCWriter()
{
}

void QCWriter::openFile(const char filename[],
                        const char software[], const char software_version[])
{
    setFilename(filename);
    startDocument();
    startElement("QC");
    writeElementString("Software", software);
    writeElementString("SoftwareVersion", software_version);
}

void QCWriter::closeFile() 
{
    endElement("QC");
    endDocument();
}

void QCWriter::writeImage(Image *image,
                          int lane, int tile, int cycle, int colour)
{
    startElement("Image");
    writeElementInt("CycleNumber",   cycle);
    writeElementInt("ChannelNumber", lane);
    writeElementInt("TileNumber",    tile);
    writeElementInt("FilterNumber",  colour);
    
    writeElementChar("Base",      (char)image->data->sequencing->base);
    writeElementInt("Deblocked",        image->data->sequencing->is_deblocked);
    writeElementString("FilePath",      image->get_filename().c_str());
    writeElementFloat("ExposureTime",   image->properties->ExposureTime);
    writeElementFloat("StagePositionX", image->properties->StageXPosition);
    writeElementFloat("StagePositionY", image->properties->StageYPosition);
    writeElementInt("ObjectCount", image->data->nobj);
    writeElementFloat("EstimatedFWHM", image->data->estimated_fwhm);
    
    // global image properties
    startElement("Intensity");
    writeElementFloat("Average", image->data->image_stats->average);
    writeElementFloat("StandardDeviation",
                                 image->data->image_stats->standard_deviation);
    writeElementFloat("Minimum", image->data->image_stats->minimum);
    writeElementFloat("Maximum", image->data->image_stats->maximum);
    writeElementFloat("SaturationFraction",
                                 image->data->image_stats->fraction_saturation);
    int i;
#if 0
    for (i = 0; i < image->data->nlocal_stats; i++) {
        ImageStatsRecord *t = &(image->data->local_image_stats[i]);
        startElement("ImageRegion");
        writeElementString("ImageRegionType", "Rectangle");
        writeElementInt("ImageRegionX1", t->x1);
        writeElementInt("ImageRegionY1", t->y1);
        writeElementInt("ImageRegionX2", t->x1 + t->nx);
        writeElementInt("ImageRegionY2", t->y1 + t->ny);
        writeElementFloat("Average", t->average);
        writeElementFloat("StandardDeviation", t->standard_deviation);
        writeElementFloat("Minimum", t->minimum);
        writeElementFloat("Maximum", t->maximum);
        writeElementFloat("SaturationFraction", t->fraction_saturation);
        endElement("ImageRegion");
    }
#endif

    endElement("Intensity");
    
    // image alignment
    startElement("UsedAlignment");
    writeElementFloat("OffsetX", image->data->usedTransform->offsetx);
    writeElementFloat("OffsetY", image->data->usedTransform->offsety);
    writeElementFloat("StretchX", image->data->usedTransform->scalex);
    writeElementFloat("StretchY", image->data->usedTransform->scaley);
    endElement("UsedAlignment");

    startElement("Alignment");
    writeElementFloat("OffsetX", image->data->transform->offsetx);
    writeElementFloat("OffsetY", image->data->transform->offsety);
    writeElementFloat("StretchX", image->data->transform->scalex);
    writeElementFloat("StretchY", image->data->transform->scaley);
    writeElementFloat("CorrelationMinimum",
                      image->data->transform->correlation_minimum);
    writeElementFloat("CorrelationMaximum",
                      image->data->transform->correlation_maximum);
    writeElementFloat("CorrelationWidth",
                      image->data->transform->correlation_width);
    for (i = 0; i < image->data->transform->nlocal; i++) {
        AlignmentRecord *t = &(image->data->transform->local_alignment[i]);
        startElement("ImageRegion");
        writeElementString("ImageRegionType", "Rectangle");
        writeElementInt("ImageRegionX1", t->firstx);
        writeElementInt("ImageRegionY1", t->firsty);
        writeElementInt("ImageRegionX2", t->firstx + t->nx);
        writeElementInt("ImageRegionY2", t->firsty + t->ny);
        writeElementFloat("OffsetX", t->offsetx);
        writeElementFloat("OffsetY", t->offsety);
        writeElementFloat("CorrelationMinimum", t->correlation_minimum);
        writeElementFloat("CorrelationMaximum", t->correlation_maximum);
        writeElementFloat("CorrelationWidth", t->correlation_width);
        endElement("ImageRegion");
    }
    endElement("Alignment");

    if (image->data->transform2) {
        startElement("Alignment");
        writeElementFloat("OffsetX", image->data->transform2->offsetx);
        writeElementFloat("OffsetY", image->data->transform2->offsety);
        writeElementFloat("StretchX", image->data->transform2->scalex);
        writeElementFloat("StretchY", image->data->transform2->scaley);
        writeElementFloat("CorrelationMinimum",
                          image->data->transform2->correlation_minimum);
        writeElementFloat("CorrelationMaximum",
                          image->data->transform2->correlation_maximum);
        writeElementFloat("CorrelationWidth",
                          image->data->transform2->correlation_width);
        endElement("Alignment");
    }

    endElement("Image");
}
