/*
 * Implement a primitive XML parser that loosely follows the SAX API.
 * No validation is done; this is just a minimal implementation for the 
 * specific purposes of the analysis software.
 *
 * Copyright (c) Solexa 2005, 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
*/
#include "XMLSAXParser.h"
#include <cstring>

XMLSAXParser::XMLSAXParser()
{
}

XMLSAXParser::~XMLSAXParser()
{
}

void XMLSAXParser::setContentHandler(XMLContentHandler *hd) 
{
    handlers.push_back(hd);
}

void XMLSAXParser::parse(FILE *fh, XMLContentHandler *hd)
{
    if (hd != 0)
        handlers.push_back(hd);
    if (handlers.size() == 0)
        return;

    /* start document */
    std::list<XMLContentHandler *>::iterator handler;
    for (handler = handlers.begin(); handler != handlers.end(); ++handler)
        (*handler)->startDocument();
    char buffer [1000];
    char text[1000];
    char tag[1000];
    /* loop over lines in file */
    while (fgets(buffer, sizeof(buffer), fh)) {
        int tagopen = 0;
        int starttag = 1;
        int tagpos = 0;
        int textpos = -1;
        int i, k;
        for (i = 0; i < (int)strlen(buffer); i++) {
            if (buffer[i] == '<') { /* tag detected */
                tagopen = 1;
                starttag = 1;
                tagpos = i;
            }
            if ((buffer[i] == '/') && tagopen && (tagpos == i-1)) {
                starttag = 0;  /* end tag detected */
                tagpos = i;
            }
            if (buffer[i] == '>') { /* tag closes */
                if (tagopen) {
                    for (k = 0; k < i-tagpos-1; k++)
                        tag[k] = buffer[k+tagpos+1];
                    tag[i-tagpos-1] = '\0';
                    if (starttag) { /* notify handler of startElement */
                        for (handler = handlers.begin();
                             handler != handlers.end(); ++handler)
                            (*handler)->startElement(tag);
                        textpos=i+1;
                    }
                    else { /* notify handler of endElement */
                        if (textpos >= 0) { /* if there was text between tags,
                                                notify handler */
                            for (k = 0; k < tagpos -textpos-1; k++)
                                text[k] = buffer[k+textpos];
                            text[tagpos-textpos-1] = '\0';
                            for (handler = handlers.begin();
                                 handler != handlers.end(); ++handler)
                                (*handler)->characters(text, 0, i-textpos-3);
                            textpos = -1;
                        }
                        for (handler = handlers.begin();
                            handler != handlers.end(); ++handler)
                            (*handler)->endElement(tag);
                    }
                } 
                tagopen = 0;
            }
        }
    }
    /* end document */
    for (handler = handlers.begin(); handler != handlers.end(); ++handler)
        (*handler)->endDocument();
}
