#!/usr/bin/env perl

# Copyright (c) 2006 Solexa
# Authors: 
# M. Parkinson, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

use warnings;
use strict;
use Cwd;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id:';
    my $cvs_version='$Revision:';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


# Plot average intensity versus cycle

my $usage=<<"END";
Usage: $0 qualityFilterPath qualityCriterion useBases readLength [baseCallDir]
[qhgDir] sigFile[s] seqFiles[s] qhgFile[s]
baseCallDir - directory whyere sigFiles and seqFiles are to be found
qhgDir - directory whyere qhgFiles are to be found
NB in useBases string: N means deblock cycle, n means other non-sequence cycle
END

die $usage unless (@ARGV>=7);


my $qualityFilter=shift(@ARGV);
my $qcCrit=shift(@ARGV);
my $deblock = shift(@ARGV);
my $readLength = shift(@ARGV);
my $numTriplets;# later=scalar(@ARGV)/3;



my ($i,$j,$cycleNum,$sigLine,$seqLine,$laneNum,$loop);
my $sigPipe;
my $seqPipe;
my $offset;
my $useBases;

my $baseCallDir=undef;
my $qhgDir=undef;
my $seqFilePath;
my $sigFilePath;
my $qhgFilePath;
my $thisCall;

my (@array,@sig,@seq,@all,@call,@tcall,@bcall);
my %lanes;
my @bases = qw(A C G T);

my %thisCallArray =
    ( 'A'=>0,
      'C'=>1,
      'G'=>2,
      'T'=>3 );
my $thisCallKey;


for($i=0;$i<@ARGV;$i++){
    print STDERR "$ARGV[$i]\n";
}

print STDERR "Read Length = $readLength";

####Going to handle deblocks here to give a pattern

if($deblock eq 'all')
{
    $useBases = 'Y' x $readLength;
} # if
elsif ($deblock eq 'odd')
{
    $useBases = 'YN' x $readLength;
    chomp($useBases);  ## last deblock not done
} # elsif
else 
{
    my $numY = 0;
    $useBases = $deblock;
    for($i=0;$i<length($deblock);$i++){
	if(substr($deblock,$i,1) =~ m/[Y]/)
	{
	    $numY++;
	}
	elsif (substr($deblock,$i,1) !~ m/[Nn]/){
	    die "deblock flag must be only YNn $deblock\n";
	}
    }

    if($numY != $readLength){
	die "The readLength ($readLength) and deblock usable bases ($numY) do not match\n";
    }
} # else


print STDERR "$useBases\n";

if (-d $ARGV[0])
{
    $baseCallDir=shift (@ARGV);
    $baseCallDir.="/" unless ($baseCallDir =~ /\/$/);
} # if

if (-d $ARGV[0])
{
    $qhgDir=shift (@ARGV);
    $qhgDir.="/" unless ($qhgDir =~ /\/$/);
} # if


if ((scalar(@ARGV)%3)!=0)
{
    warn "Expecting triplets of seq,sig,qhg files, but num of files (",
    scalar(@ARGV), ") is not a multiple of 3!";
    exit (0);
} # if

$numTriplets=scalar(@ARGV)/3;
print STDERR "Assuming that the input consists of $numTriplets of  sig,seq,qhg triplets\n";


for($loop=0;$loop<$numTriplets;$loop++)
{
    $sigFilePath=$ARGV[$loop];
    $seqFilePath=$ARGV[$numTriplets+$loop];
    $qhgFilePath=$ARGV[$numTriplets+$numTriplets+$loop];
    if (defined($baseCallDir))
    {
	$sigFilePath=$baseCallDir.$sigFilePath;
	$seqFilePath=$baseCallDir.$seqFilePath;
    } # if
    if (defined($qhgDir))
    {
	$qhgFilePath=$qhgDir.$qhgFilePath;
    } # if
    

    print STDERR "Filtering $sigFilePath AND $seqFilePath with $qhgFilePath\n";

    $sigPipe="cat $sigFilePath | perl $qualityFilter \'$qcCrit\' $qhgFilePath |";
    $seqPipe="cat $seqFilePath | perl $qualityFilter \'$qcCrit\' $qhgFilePath |";
    
    print  STDERR "$sigPipe\n";

    print  STDERR "$seqPipe\n";

    
    open (PIPE, $sigPipe) 
        || die "Error: failed to open pipe $sigPipe: $!";
    
    open (PIPE2, $seqPipe) 
        || die "Error: failed to open pipe $seqPipe: $!";
    
    
    
    while (defined($sigLine=<PIPE>)) {
	$seqLine = <PIPE2>;


	@sig = split('\t',$sigLine);
	@seq = split('\t',$seqLine);
	

	$lanes{$sig[0]}++;


	## Going to do some error checks here as 1st few columns should be same

	if($sig[0] ne $seq[0]){
	    die "The sig and seq lines are not from same lane $sig[0] $seq[0]\n";
	}
	
	
	if($sig[1] ne $seq[1]){
	    die "The sig and seq lines are not from same tile $sig[1] $seq[1]\n";
	}
	

	if($sig[2] ne $seq[2]){
	    die "The sig and seq lines don't have same x-coord $sig[2] $seq[2]\n";
	}
	
	if($sig[3] ne $seq[3]){
	    die "The sig and seq lines don't have same y-coord $sig[3] $seq[3]\n";
	}
	



	## This needs to be back compatible so will not estimate read length

	$cycleNum  = 1;

	for($j=0;$j<length($useBases);$j++){
	    ## Ignore Noise in older image analysis versions
	    
	    if(substr($useBases,$j,1) eq 'Y'){
		### Cycle information
		@array = split(' ',$sig[$j+4]);
		

		$all[$sig[0]][$cycleNum]{A} += $array[0];
		$all[$sig[0]][$cycleNum]{C} += $array[1];
		$all[$sig[0]][$cycleNum]{G} += $array[2];
		$all[$sig[0]][$cycleNum]{T} += $array[3];
		$all[$sig[0]][$cycleNum]{Count}++;

		## Now do the base calls
		# add variable to avoid incessant calls to substr
		$thisCall=substr($seq[4],$j,1);
		
		$bcall[$sig[0]][$cycleNum]{$thisCall}++;
		$bcall[$sig[0]][$cycleNum]{Count}++;

		# next bit replaces a few 'elsifs', hopefully quicker
		if($thisCall =~ m/[ACGT]/)
		{	
		    $call[$sig[0]][$cycleNum]{$thisCall} 
		    += $array[$thisCallArray{$thisCall}];
		    $thisCallKey='Count'.$thisCall;
		    $call[$sig[0]][$cycleNum]{$thisCallKey}++;
		} # if
		
		if(substr($useBases,($j+1),1) =~ m/[Y]/)
		{
		    ## next cycle isn't a deblock (need to keep in phase)
		    ## if deblock with increment in that part of code
		    $cycleNum++;
		} # if
	    } # if
	    elsif(substr($useBases,$j,1) eq 'n')
	    {
		## unused enzymology
	    } # elsif
	    else 
	    {
		##  going to do a deblock 
		@array = split(' ',$sig[$j+4]);

		$tcall[$sig[0]][$cycleNum]{A} += $array[0];
		$tcall[$sig[0]][$cycleNum]{C} += $array[1];
		$tcall[$sig[0]][$cycleNum]{G} += $array[2];
		$tcall[$sig[0]][$cycleNum]{T} += $array[3];
		$tcall[$sig[0]][$cycleNum]{Count}++;
		$cycleNum++;
	    }
	}
    } # while
    close (PIPE)
        || die "Error: failed to close pipe $sigPipe: $!";
    
    close (PIPE2)
        || die "Error: failed to close pipe $seqPipe: $!";
}

### Now need to output values 
## print a header line

print "#Lane\tCycle\tAll A\tAll C\tAll G\tAll T\tCall A\tCall C\tCall G\tCall T\tBase A\tBase C\tBase G\tBase T\tDeblock A\tDeblock C\tDeblock G\tDeblock T\n";

foreach $laneNum (sort {$a <=> $b} keys %lanes){
    ## only going to print out relevant lines
    

    for($i=1;$i<($readLength+1);$i++){
	
	print "$laneNum\t$i\t";

	## Now for All bases
	$all[$laneNum][$i]{Count} = 0 unless defined ($all[$laneNum][$i]{Count});
	foreach (@bases){

	    $all[$laneNum][$i]{$_} = 0 unless defined ($all[$laneNum][$i]{$_});

	    if ($all[$laneNum][$i]{Count} != 0){
		$all[$laneNum][$i]{$_} /= $all[$laneNum][$i]{Count};
	    }
	    printf ("%0.1f\t",$all[$laneNum][$i]{$_});
	}

	## Now for Call bases
	$call[$laneNum][$i]{CountA} = 0 unless defined ($call[$laneNum][$i]{CountA});
	$call[$laneNum][$i]{CountC} = 0 unless defined ($call[$laneNum][$i]{CountC});
	$call[$laneNum][$i]{CountG} = 0 unless defined ($call[$laneNum][$i]{CountG});
	$call[$laneNum][$i]{CountT} = 0 unless defined ($call[$laneNum][$i]{CountT});


	$call[$laneNum][$i]{A} = 0 unless defined ($call[$laneNum][$i]{A});

	if ($call[$laneNum][$i]{CountA} != 0){
	    $call[$laneNum][$i]{A} /= $call[$laneNum][$i]{CountA};
	}
	printf ("%0.1f\t",$call[$laneNum][$i]{A});


	$call[$laneNum][$i]{C} = 0 unless defined ($call[$laneNum][$i]{C});

	if ($call[$laneNum][$i]{CountC} != 0){
	    $call[$laneNum][$i]{C} /= $call[$laneNum][$i]{CountC};
	}
	printf ("%0.1f\t",$call[$laneNum][$i]{C});

	$call[$laneNum][$i]{G} = 0 unless defined ($call[$laneNum][$i]{G});

	if ($call[$laneNum][$i]{CountG} != 0){
	    $call[$laneNum][$i]{G} /= $call[$laneNum][$i]{CountG};
	}
	printf ("%0.1f\t",$call[$laneNum][$i]{G});

	$call[$laneNum][$i]{T} = 0 unless defined ($call[$laneNum][$i]{T});

	if ($call[$laneNum][$i]{CountT} != 0){
	    $call[$laneNum][$i]{T} /= $call[$laneNum][$i]{CountT};
	}
	printf ("%0.1f\t",$call[$laneNum][$i]{T});


	#### Now for the % called bases

	$bcall[$laneNum][$i]{Count} = 0 unless defined ($bcall[$laneNum][$i]{Count});
	foreach (@bases){

	    $bcall[$laneNum][$i]{$_} = 0 unless defined ($bcall[$laneNum][$i]{$_});

	    if ($bcall[$laneNum][$i]{Count} != 0){
		$bcall[$laneNum][$i]{$_} /= $bcall[$laneNum][$i]{Count};
		$bcall[$laneNum][$i]{$_} *= 100;
	    }
	    printf ("%0.1f\t",$bcall[$laneNum][$i]{$_});
	}


	## Now for Deblock bases
	$tcall[$laneNum][$i]{Count} = 0 unless defined ($tcall[$laneNum][$i]{Count});
	foreach (@bases){

	    $tcall[$laneNum][$i]{$_} = 0 unless defined ($tcall[$laneNum][$i]{$_});

	    if ($tcall[$laneNum][$i]{Count} != 0){
		$tcall[$laneNum][$i]{$_} /= $tcall[$laneNum][$i]{Count};
	    }
	    printf ("%0.1f\t",$tcall[$laneNum][$i]{$_});
	}
	
	print "\n";
    } # readLength Loop
} # lane loop

