#!/usr/bin/env perl

# Copyright (c) 2006 Solexa
# Authors: 
# M. Parkinson, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

use warnings;
use strict;
use Cwd;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: Signal_Means_Tile.pl,v 1.6 2006/07/27 13:59:46 tc Exp $';
    my $cvs_version='$Revision: 1.6 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


# Plot average intensity versus sequencing cycle

die "Usage: $0 qualityFilterPath qualityCriterion deblock read_length sigFile qhgFile"
   unless (@ARGV==6);

my $numChannels=4;
my $qualityFilter=shift(@ARGV);
my $qcCrit=shift(@ARGV);
my $deblock = shift(@ARGV);
my $readLength = shift(@ARGV);
my $sigFile = shift(@ARGV);
my $qhgFile = shift(@ARGV);

my ($i,$j,$laneNum,$sigPipe,$cycleNum,$useBases,$sigLine);
my (@all,@array,@sig);
my @bases = qw(A C G T);

#print STDERR "$0: readLength=$readLength\n";

####Going to handle deblocks here to give a pattern

if($deblock eq 'all'){
    $useBases = 'Y' x $readLength;
}
elsif ($deblock eq 'odd'){
    $useBases = 'YN' x $readLength;
}
else {
    my $numY = 0;
    $useBases = $deblock;
    for($i=0;$i<length($deblock);$i++){
	if(substr($deblock,$i,1) =~ m/[Yy]/){
	    $numY++;
	}
	elsif (substr($deblock,$i,1) !~ m/[Nn]/){
	    die "deblock flag must be only YyNn $deblock\n";
	}
    }

    if($numY != $readLength){
	warn "Warning: readLength ($readLength) does not match number of usable bases ($numY) specified in $useBases";
    }
}

#print STDERR "$0: useBases=$useBases\n";

#print STDERR "Filtering $sigFile with $qhgFile\n\n";

$sigPipe="cat $sigFile | perl $qualityFilter \'$qcCrit\' $qhgFile |";


#print  STDERR "$sigPipe\n";

open (PIPE, $sigPipe) 
    || die "Error: failed to open pipe $sigPipe: $!";


while (defined($sigLine=<PIPE>)) {

    @sig = split('\t',$sigLine);

    $laneNum = $sig[0];
#	print STDERR $sigLine;
    ## This needs to be back compatible so will not estimate read length

    $cycleNum  = 1;
    
    for($j=0;$j<length($useBases);$j++)
    {
	## Ignore Noise in older image analysis versions
	
	if(substr($useBases,$j,1) =~ m/[Yy]/)
	{
	    ### Cycle information
	    @array = split(' ',$sig[$j+4]);
	    
	    if (@array==0)
	    {
		warn "Failed to find intensity values for cycle $j\n";
		last;
	    }
	    $all[$cycleNum]{A} += $array[0];
	    $all[$cycleNum]{C} += $array[1];
	    $all[$cycleNum]{G} += $array[2];
	    $all[$cycleNum]{T} += $array[3];
	    $all[$cycleNum]{Count}++;
	    last if ($cycleNum++==$readLength);
	    
	} # if
    } # for
} # while

close (PIPE)
    || die "Error: failed to close pipe $sigPipe: $!";

unless (defined($laneNum))
{
    # No intensity info was found: output a set of zero intensities 
    warn "Warning: no intensity information obtained from pipe '$sigPipe'";
    # Set laneNum to -1 rather than guess
    $laneNum=-1;
} # unless


print "#Lane\tCycle\tAll A\tAll C\tAll G\tAll T\n";

for($i=1;$i<($readLength+1);$i++){
    
    print "$laneNum\t$i\t";

    ## Now for All bases
    $all[$i]{Count} = 0 unless defined ($all[$i]{Count});
    foreach (@bases){

	$all[$i]{$_} = 0 unless defined ($all[$i]{$_});

	if ($all[$i]{Count} != 0){
	    $all[$i]{$_} /= $all[$i]{Count};
	}
	printf ("%0.1f\t",$all[$i]{$_});
    }
    print "\n";
}

