#!/usr/bin/env perl

# File: buildCoverage.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description: build coverage map from sets of alignment data

use warnings;
use strict;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: buildCoverage.pl,v 1.7 2007/01/30 16:57:37 tc Exp $';
    my $cvs_version='$Revision: 1.7 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

print "#RUN_TIME ". scalar(localtime)."\n";
print "#SOFTWARE_VERSION $id\n";


die "Usage: buildCoverage.pl genomeName [scoreThreshold]"
    unless ((@ARGV==1)||(@ARGV==2));

my $genomeName=$ARGV[0];
my $scoreThreshold=-100000000;
$scoreThreshold=$ARGV[1]
    unless (@ARGV==1);
my @bases=('A', 'C', 'G', 'T');
my $isDos=0;
my $line;

open( FILE, "$genomeName") || die "problem opening $genomeName";

$line=<FILE>; # ignore 1st line

if ($line=~/\r\n$/)
{
    print STDERR "WARNING: $genomeName looks like a DOS format text file, will attempt to compensate\n";
    $isDos=1;
} # if

my %baseCall;
my @sp;
my $seq;
my $genome="?"; # to make everything index from 1
my $i;
my $max=0;
my $maxNonWild=0;
my $maxWild;
my $cons;
my $adjPos; # adj for circularization
my $genomeLength;
my $base;

while (<FILE>)
{
    s/\r\n$/\012/g if ($isDos);
    chomp;
    $genome .=  $_;
} # while
$genomeLength=length($genome)-1; 
# -1 to account for '?' at the beginning
$genome=uc($genome);

# set arrays to correct size (saves multiple memory reallocations)
for $base (@bases)
{
    $baseCall{$base}[$genomeLength]=0;
}


while (<STDIN>)
{
    if (/^\#/) { print; next; } 
    chomp;
    @sp=split(' ', $_);
    next if (@sp==3);
    die unless (@sp>=6);
    next if ($sp[1]<$scoreThreshold);
    if ($sp[4] eq 'R')
    {
	$seq=reverse($sp[0]);
	$seq =~ tr/ACGTacgt/TGCATGCA/;
    } #if
    else
    {
	$seq=uc($sp[0]);
    } #else
    for ($i=0; $i<length($seq);$i++)
    {
	$adjPos=$sp[3]+$i;
	$adjPos-=$genomeLength if ($adjPos>$genomeLength);
	$base=substr($seq,$i,1)++;
	$baseCall{$base}[$adjPos]++ if ($base=~/[ACGT]/);
    } #for
} #while

my $genomeBase;

for ($i=1; $i<=$genomeLength; $i++)
{
    $genomeBase=substr($genome, $i, 1);
    print $i, " ", $genomeBase;
    $genomeBase=uc($genomeBase);
    $max=0;
    $maxNonWild=0;
    $cons="X";
    foreach $base (@bases)
    {
	$baseCall{$base}[$i]=0 unless defined ($baseCall{$base}[$i]);
	print "\t$baseCall{$base}[$i]";
	if ($baseCall{$base}[$i]>$max)
	{
	    $cons=$base;
	    $max=$baseCall{$base}[$i];
	} #if
	elsif (($max!=0)&&($baseCall{$base}[$i]==$max))
	{ # a tie
	    $cons.=$base;
	} #elsif
	next if ($base eq $genomeBase);
	if ($baseCall{$base}[$i]>$maxNonWild)
	{
	    $maxNonWild=$baseCall{$base}[$i];
	} #if
    } #foreach
    $maxWild=$baseCall{$genomeBase}[$i];
    $maxWild=0 unless (defined($maxWild));
    print "\t$cons\t$maxWild\t$maxNonWild\n";
} #for i
