#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  convertQuality.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Takes a prb file and converts each line to a quality string
# Conversion factor is chr=ASCII(64+QV) (NB that's 64 decimal)
# so smallest possible QV of -5 gets called ASCII 59=';' 
# QV = 0  gets ASCII 64 = '@'
# QV = 30 gets ASCII 94 = '^'

use warnings;
use strict;
use File::Basename;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: convertQuality.pl,v 1.4 2006/12/05 16:03:45 tc Exp $';
    my $cvs_version='$Revision: 1.4 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


my $usage=
"Usage: $0 [--useBases useBases] [--numeric|--symbolic] readLength prbFile[s]";


my $useBases=undef;
my $isNumeric=0;

die $usage unless (@ARGV>=2);

if ($ARGV[0] eq '--useBases')
{
    shift (@ARGV);
    die $usage unless (@ARGV>=3);
    $useBases=shift(@ARGV); 
} # if

if (($ARGV[0] eq '--numeric')||($ARGV[0] eq '--symbolic'))
{
    $isNumeric=1 if ($ARGV[0] eq '--numeric');
    shift (@ARGV);
}




die $usage unless (@ARGV>=2);

my $readLength=shift(@ARGV);


my @baseIndex;

unless (defined($useBases))
{
    $useBases='all';
}

my $i;
my $j;
my $qv;

my $qstring;


sub parseUseBases
{
    my ($useBases)=@_;
    if ($useBases eq "all")
    {
        $useBases= "Y" x $readLength;
    } # if
    elsif ($useBases eq "odd")
    {
        $useBases= "YN" x ($readLength);
        $useBases=substr($useBases,0,length($useBases)-1);
    } # elsif
    else
    {
        die "Use bases string $useBases must be odd, all or YNYN..."
            unless ($useBases=~/^[YNyn]+$/);
        $useBases = uc ($useBases);
    } # else

    my @ub=split(//,$useBases);
  #  my @baseIndex;
    for ($i=0;$i<@ub;$i++)
    {
        push (@baseIndex, $i) if ($ub[$i] eq "Y"); 
    } # if

    die "Need $readLength bases of base information, only ", 
    scalar(@baseIndex), " specified in $useBases"
        unless (scalar(@baseIndex)>=$readLength);
    @baseIndex=splice(@baseIndex,0,$readLength);
} # sub parseUseBases


sub max
{
    my $max=$_[0]->[0];
    for (@{$_[0]})
    {
        $max=$_ if ($_>$max);
    }
    return $max;
} # sub max


parseUseBases($useBases);


#print scalar(@baseIndex), " $baseIndex[0] $baseIndex[-1]\n"; die;
my @s1;
my @s3;

for my $file (@ARGV)
{

    open (LINE, $file) 
	|| die "Failed to open $file: $!";

    while (<LINE>)
    { 
	$qstring='';
	@s1=split('\t',$_);
	for ($i=0;$i<scalar(@baseIndex);$i++)
	{
#            print "$i $baseIndex[$i] $s1[$baseIndex[$i]]\n";
	    @s3=split(' ',$s1[$baseIndex[$i]]);
	    unless (@s3==4)
	    {
		print STDERR basename($0), ": Error parsing line: $_";
		print STDERR basename($0),
		": can't find quality values for cycle ",
		$baseIndex[$i]+1, " in file ${file} "; 
		if (defined($s1[$baseIndex[$i]]))
		{
		    print STDERR
			"- could not parse field $s1[$baseIndex[$i]]\n";
		} # if
		else
		{
		    print STDERR 
			"- only ",scalar(@s1), 
			" cycles of quality values present on line\n";
		} # else			
		exit (-1);
	    }
	    $qv=max(\@s3);
	    $qstring.=(($isNumeric)?" $qv":chr(int(64+$qv)));
	} # for i
	$qstring=~s/^\ //;
	printf "%s\n", $qstring;
    } # while  
} # for file
