#!/usr/bin/env perl
use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

# File: convertToFasta.pl
# Copyright (c) Solexa Limited 2006-7
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Simple script to convert "Solexa-style" sequence files into
# Fasta-style format

# Usage: 
# [ cat seq1.txt seq2.txt... ] | ./convertToFasta.pl 
# [-useBases (all|odd|YNYN...) ] [ seq3.txt seq4.txt ... ]

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: convertToFasta.pl,v 1.5 2007/02/22 09:52:49 tc Exp $';
    my $cvs_version='$Revision: 1.5 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope



my $seqFile;
my $seqFileNoPath;
my $seq;
my $useBases="all";
my $unp=undef;
my @s;
my $l;
my $seqIn;
my $seqOut;
my $shortRunFolderName;

if ((@ARGV>=2)&&($ARGV[0] eq "-useBases"))
{
    shift (@ARGV);
    $useBases=shift(@ARGV);
    die "Could not parse -useBases: must be all, odd or all Ys and Ns" 
        unless (($useBases eq "all")
                ||($useBases eq "odd")
                ||($useBases =~ /^[YyNn]+$/));
} # if


die "Usage: $0 [-useBases (all|odd|YNYN...)] seq1.txt seq2.txt ..."
    unless (@ARGV>0);

$"=':';

#$"='_';

while ($seqFile=shift(@ARGV))
{
    open (FILE, $seqFile)
        || die "Problem opening $seqFile: $!";

    my $bustardFolder=File::Spec->rel2abs(dirname($seqFile));
    @s=split('/',$bustardFolder);
  #  print scalar(@s), "$bustardFolder\n"; die;
    my $shortRunFolderName=getShortRunFolderName($s[-4]); 

    $seqFileNoPath=basename($seqFile);

    while ($l=<FILE>)
    {
#    print $l;
        chomp($l);
        @s=split(' ',$l);
        $seqIn=pop(@s);

        unless (defined($unp))
        {
            if ($useBases eq "all")
            {
                for (my $i=0;$i<length($seqIn); $i++)
                {
                    $unp.="Y";
                } # for    
            } # if
            elsif ($useBases eq "odd")
            {
                for (my $i=0;$i<length($seqIn); $i+=2)
                {
                    $unp.="YN";
                } # for   
                $unp=substr($unp,0,length($seqIn));
            } # if
            else
            {
                $unp=$useBases;
            } # else
            $unp=~tr/YyNn/ccxx/;
            #    print "$unp --\n";
        } # if
    
        $seqOut="";
        map {$seqOut.=chr($_) } unpack $unp, $seqIn;
        $seqOut=~s/\./N/g;
        print ">${shortRunFolderName}:@s\n$seqOut\n";
 #       print ">@s\n$seqOut\n";
#        print ">$seqFileNoPath\n$seqOut\n";
  
    } # while FILE...

    close (FILE);

} # while $seqFile...
