#!/usr/bin/env perl


## This program takes the individual error plots and combines them
## into a single htm page which links to original files
## The page format has been designed by Tony Cox
# Copyright (c) Solexa 2006
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

### The program is called using directory name

use warnings;
use strict;
use Cwd;

# To add support for new thumbnail types, add an appropriate entry 
# to %suffix (filename suffix of files to thumbnail)
# and %title (title for generated thumbnail page)

# hash of suffixes
my %suffix=
( "error"=>"_rescore.png",
  "base"=>"_base.png",
  "call"=>"_call.png",
  "all"=>"_all.png",
  "perfect"=>"_errors.png",
  "hist"=>"_hist.png",
  "info"=>"_info.png" );

# hash of page titles
my %title=
( "error"=>"Errors",
  "base"=>"Base",
  "call"=>"Calls",
  "all"=>"All",
  "perfect"=>"Error Curves",
  "hist"=>"Quality Score Histograms",
  "info"=>"Information Content" );
            
my $version;
my $id;
{
    my $cvs_id='@(#) $Id: create_error_thumbnails.pl,v 1.7 2007/01/18 19:32:16 km Exp $';
    my $cvs_version='$Revision: 1.7 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope



my ($file,$i,$j,$expt_name,$val,$k, $suffixToFind, $titleToUse);
my $mapping = 0;  # going to use this to work out whether manteia, 1 column or 2 column
my (@array);
my (%lane,%tile,%results, %lanesPresent, %tilesPresent);




if ((@ARGV!=1)&&(@ARGV!=2))
{
    $"='/';
    @array=sort keys %suffix;
    print STDERR 
        "Usage: $0 create_error_thumbnails.pl type [dir], where\n";
    print STDERR 
        "type = @array\n"; 
    print STDERR 
        "dir  = path of GERALD dir or omit to use current working dir\n";
    die;
} # if

if (defined($suffix{$ARGV[0]}))
{
    $suffixToFind=$suffix{$ARGV[0]};
    $titleToUse=$title{$ARGV[0]};
} # if
else
{
    $"='/';
    @array=sort keys %suffix;
    print STDERR "$ARGV[0] not one of @array, assuming it's a suffix\n";
    $suffixToFind=$ARGV[0];
    unless ($suffixToFind=~/\.png$/)
    {
        $suffixToFind.='.png';
    } # unless
    unless ($suffixToFind=~/^_/)
    {
        $suffixToFind='_'.$suffixToFind;
    } # unless

    $titleToUse="Files of type *$ARGV[0]";
} # else


$ARGV[1]=getcwd() unless (@ARGV==2);



## Look for experimental number

$expt_name=undef;
if ($ARGV[1] =~ m#/(\d{6}_[^/_]+_\d{4}[^/]*)/#) {
    $expt_name = $1;
}
if ($ARGV[1] =~ m#/(\d{8})/#) {
    $expt_name = $1;
}

warn "Could not parse experiment name from $ARGV[1]"
    unless (defined($expt_name));

$expt_name="" unless (defined($expt_name));


## Define output file name


print  "<html>\n<head>\n";

print  "<!--RUN_TIME ".scalar(localtime)."-->\n";
print  "<!--SOFTWARE_VERSION $id-->\n";


print  qq(</head>)."\n";

print  qq(<title>$expt_name $titleToUse</title>);
print  qq(<h1 align="center">$expt_name $titleToUse</h1>);


print "\n<body>\n";



## Starting the html output with the body name


opendir (DIR,$ARGV[1]) or die "can't open $ARGV[1]\n";


while (defined($file = readdir (DIR))){
#    next if ($file !~ m/_rescore.png/);

 #   print STDERR "$file\n";


    next unless ($file=~/s_([1-9])_([0-9]{4})${suffixToFind}/);

    $results{$1}{$2} = $file;
    $lane{$1}++;
    $tile{$2}++;


#    next if ($file!~m/\d{4}${suffixToFind}$/);

#    @array = split("_",$file);
    
#    $results{$array[1]}{$array[2]} = $file;
#    $lane{$array[1]}++;
#    $tile{$array[2]}++;

# Next bit attempts to make the web page layout mimic the configuration
# of the tiles on the chip. Difficult to do this while a) chip layout is
# subject to change and b) there is no reliable way of determining what
# the layout is from the instrument output. Until these issues are resolved,
# always print tiles of each lane as one long column.  

#    if($array[2] < 8){
#	$val = 0;
#    }
#    elsif ($array[2] < 76){
#	$val = 1;
#    }
#    elsif ($array[2] < 151){
#	$val = 2;
#    }
#    else {
#	print "Unknown Tile Configuration tile = $val\n";
#	die;
#    }
#
#    if($val > $mapping){
#	$mapping = $val;
#    }
    
} # while


$mapping = 1;

print  qq(<table border="1" cellpadding="5"><tr><td><b>Tile</b></td>)."\n";
foreach (sort {$a<=>$b} keys %lane)
{
    if($mapping < 2)
    {
	print  qq(<td><b>Lane $_</b></td>)."\n";
    } # if
    if($mapping == 2){
	## need extra columns
	print  qq(<td><b>Lane $_ A</b></td>)."\n";
	print  qq(<td><b>Lane $_ B</b></td>)."\n";
    } # if
} # foreach

print  qq(</tr>)."\n";
if($mapping < 2)
{
    foreach $i (sort {$a<=> $b} keys %tile)
    {
	print  qq(<tr><td><b>$i</b></td>)."\n";
	foreach $j (sort {$a<=>$b} keys %lane)
	{
	    if(defined $results{$j}{$i})
	    {
		print  qq(<td><a href="$results{$j}{$i}"> <img height=84 width=84 src="$results{$j}{$i}"></a></td>)."\n";
	    } # if
	    else 
	    {
		print  qq(<td>&nbsp;</td>)."\n";
	    } # else
	} # foreach
	print  qq(</tr>)."\n";
    } # foreach

} # if
else 
{
    # Attempts to do two sublanes per lane. Needs rewriting

    for ($i=1;$i<76;$i=$i+2)
    { 
	$k = $i + 1;
	if(length($i) == 1)
	{
	    $i = '000'.$i;
	}
	elsif (length($i) == 2)
	{
	    $i = '00'.$i;
	}
	elsif (length($i) == 3)
	{
	    $i = '0'.$i;
	}

	if(length($k) == 1)
	{
	    $k = '000'.$k;
	}
	elsif (length($k) == 2)
	{
	    $k = '00'.$k;
	}
	elsif (length($k) == 3)
	{
	    $k = '0'.$k;
	}
	if((defined $tile{$i}) ||(defined $tile{$k})) 
	{
	    print  qq(<tr><td><b>$i / $k </b></td>)."\n";

	    foreach $j (sort {$a<=>$b} keys %lane)
	    {
		if(defined $results{$j}{$i})
		{
		    print  qq(<td><a href="$results{$j}{$i}"> <img height=84 width=84 src="$results{$j}{$i}"></a></td>)."\n";
		} # if
		else 
		{
		    print  qq(<td>&nbsp;</td>)."\n";
		} # else
		if(defined $results{$j}{$k})
		{
		    print  qq(<td><a href="$results{$j}{$k}"> <img height=84 width=84 src="$results{$j}{$k}"></a></td>)."\n";
		} # if
		else 
		{
		    print  qq(<td>&nbsp;</td>)."\n";
		} # else
	    } # foreach $j
	    print  qq(</tr>)."\n";
	} # if
    } # for i
} # else


print qq(</table>)."\n";
print qq(</body>)."\n";
print qq(</html>)."\n";


