#!/usr/bin/env perl

# File: plotCoverage.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces coverage plots from files output from buildCoverage.pl
# Assumes filenames end in .txt.
# For each fileName.txt, puts an associated fileName.pdf in same directory
# Needs:
# gnuplot (assumed at $pathGnuplot) to create the graph
# convert (part of ImageMagick, assumed at $pathConvert) to convert to pdf

use warnings;
use strict;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotCoverage.pl,v 1.4 2006/07/27 13:59:46 tc Exp $';
    my $cvs_version='$Revision: 1.4 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope

my $pathGnuplot="gnuplot";
my $pathConvert="convert";


my $name;
my $outName;
my $tmpName;
my @F;
my $numLines=0;
my $smoothThresh=10000;
my $smooth="";
my $maxCov=0;

die "Usage: $0 cov1.txt [ cov2.txt ... ] "
    unless (@ARGV>=1);
while ($name=shift)
{
    open (IN, $name) || die "Failed to open $name: $!";
    while (<IN>)
    {
        next if (/^\#/);
        $numLines++;
        @F=split;
        die "Could not parse line $numLines: $_\n" unless (@F==9);
        $maxCov+=($F[7]+$F[8]);
    } #while
    close (IN);
    $maxCov/=$numLines;
    $maxCov*=2;
    $smooth=" smooth bezier " if ($numLines>$smoothThresh);
    $outName=$name;
    $outName=~s/\.txt/\.png/;
    die if ($outName eq $name);

    $tmpName="/tmp/tmp-plotCoverage-$>-$$";
    open (OUT, ">$tmpName.txt") || die "$tmpName";
    print OUT << "END";
#    set nokey;
    set xrange[1:$numLines];
    set yrange[0:$maxCov];
#    set output "$tmpName.png";
#    set terminal png color;
    set size 1.4,0.6;
    set terminal postscript portrait color solid;
    set title "Coverage plot of file $name" -1;
    set xlabel "Genome position (bp)" -1;
    set ylabel "Coverage (bp)";
    plot '$name' using 1:9 $smooth title 'error' with lines lw 2, '$name' using 1:8 $smooth title 'correct' with lines lw 2;
END
    close (OUT);
#    system ("$pathGnuplot $tmpName.txt");
#    system ("$pathConvert $tmpName.png $outName");
    system ("$pathGnuplot $tmpName.txt > $tmpName.ps");
    system ("$pathConvert $tmpName.ps $outName");
} #while

unlink "$tmpName.txt", "$tmpName.ps";

