#!/usr/bin/env perl

# File: plotErrorRate.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces plots of error rates vs cycle from files output by score.pl
# Assumes filenames end in .txt.
# For each fileName.txt, puts an associated fileName.pdf in same directory
# Needs:
# gnuplot (assumed at $pathGnuplot) to create the graph
# convert (part of ImageMagick, assumed at $pathConvert) to convert to pdf

use warnings;
use strict;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotErrorRate.pl,v 1.5 2006/07/27 13:59:46 tc Exp $';
    my $cvs_version='$Revision: 1.5 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope

my $pathGnuplot="gnuplot";
my $pathConvert="convert";


my $name;
my $outName;
my $tmpName;
my $tmpNameData;
my $tagString="Breakdown of errors by cycle"; # look for this in file
#$tagString=~s/\ /\\\ /g;
#print $tagString, "\n"; die;
my @a;
my $foundStart;

die "Usage: $0 scoreFile.txt [ outFile.png ] "
    unless ((@ARGV==1)||(@ARGV==2));

$name=$ARGV[0];
#while ($name=shift)
#{
    $foundStart=0;

if (@ARGV==2)
{
    $outName=$ARGV[1];
} # if
else
{
    $outName=$name;
    $outName=~s/\.txt/\.png/;
    die if ($outName eq $name);
} # else

$tmpName="/tmp/tmp-plotError-$>-$$";
open (IN, $name) 
    || die "Problem opening $name: $!";
open (OUT, ">$tmpName.dat")
    || die "Problem opening $tmpName.dat for writing: $!";
while (<IN>)
{
    chomp;
    if ($foundStart==1)
    {
	@a=split;
	last unless ((@a=split)==3);
	print OUT "$_\n";
    } #if
    elsif ($_ eq $tagString)
    {
	die unless ($foundStart==0);
	die unless defined(<IN>); # skip a line
	$foundStart=1;
    } #elsif
} #while
close (IN);
close (OUT);
#    die "Did not find data in file" unless ($foundStart==1);

# If no data found, create a blank file and continue 
# (rather than stop pipeline)
unless ($foundStart==1)
{
    warn "Did not find data in file $name, creating blank file $outName"; 
    open (OUT, ">$outName") 
	|| die "Problem creating blank file $outName: $!";
    close (OUT);
    exit (0);
} # unless

open (OUT, ">$tmpName.txt") 
    || die "Problem opening $tmpName.txt for writing: $!";

# 1. "set output" doesn't work in multiplot mode, have to pipe
# the output to a file instead
# 2. Would like to autoscale the x axes, probably need a newer version
# of gnuplot
    print OUT << "END";
#    set terminal png color;
    set terminal postscript portrait color solid;
    set nokey;
#    set yrange [0:];
    set yrange [0:40];
    set boxwidth 0.6;
    set size 1.0, 1.0;
    set origin 0.0, 0.0;
    set multiplot;
    set size 1.0, 0.5;
    set origin 0.0, 0.5;
    set title "Error rates by cycle for file $name";
#    set xlabel "Cycle Number" -1;
    set ylabel "Percent";
    plot '$tmpName.dat' using 1:2 title 'error rate' with impulses lt 1 lw 24;    
    set origin 0.0, 0.0;
    set title "Occurrence of blank bases by cycle for file $name";
    set xlabel "Cycle Number" -1;
    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
    set nomultiplot;

END

close (OUT);
system ("$pathGnuplot $tmpName.txt > $tmpName.ps");
system ("$pathConvert $tmpName.ps $outName");
#} #while
unlink "$tmpName.dat", "$tmpName.txt", "$tmpName.ps";

