#!/usr/bin/env perl

# File: plotIntensity.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces an example intensity by cycle plot from an intensity 
# Needs:
# gnuplot (assumed at $pathGnuplot) to create the graph
# convert (part of ImageMagick, assumed at $pathConvert) to convert to png

use warnings;
use strict;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotIntensity_tiles.pl,v 1.5 2006/07/27 13:59:46 tc Exp $';
    my $cvs_version='$Revision: 1.5 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope

my $pathGnuplot="gnuplot";
my $pathConvert="convert";

my $tmpName;
my $tmpNameData;
my ($i,$j,$k,$outName,$displayName,$total,$title,$deblock);
my (@array,@intensities,@bases);
my %lanes;

my $max = 0;
my $setmax = 0;

my $width=3;
my ($file,$directory);
my @s;
my @total;
my $count;


sub draw_graphs {
    my ($lane,$tmpName,$outName,$title) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:$max];
set ytics 500;
  
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
set xlabel "Cycle Number" -1;
set ylabel "Intensity";
plot '$tmpName.dat' using 1:2 title 'A' with linespoints lt 9 lw $width pt 7 ps 2, '$tmpName.dat' using 1:3 title 'C' with linespoints lt 1 lw $width pt 7 ps 2,    '$tmpName.dat' using 1:4 title 'G' with linespoints lt 3 lw $width pt 7 ps 2, '$tmpName.dat' using 1:5 title 'T' with linespoints lt 2 lw $width pt 7 ps 2;  
  
#    set origin 0.0, 0.0;
#    set title "Occurrence of blank bases by cycle ";
#    set xlabel "Cycle Number" -1;
#    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
#    set nomultiplot;

END
    close (OUT);
    system ("$pathGnuplot $tmpName.txt > $tmpName.ps");
    system ("$pathConvert $tmpName.ps $outName");
    unlink "$tmpName.dat", "$tmpName.txt", "$tmpName.ps";
}



die "Usage: $0 Tile_all.txt Tile_all.png\n"
    unless (@ARGV==2);

open (IN, $ARGV[0]) 
    || die "Problem opening $file: $!";
while (<IN>)
{
  
    next if($_ =~ m/\#/);
    push (@intensities,[split('\t',$_)]);

}
close(IN);



### Going to start a loop for plotting graphs now

@array = split('/',$ARGV[1]);
$title = $array[@array-1];

### Going to do ALL 
     
$tmpName="/tmp/tmp-plotIntensity-$>-$$";
open (OUT, ">$tmpName.dat")
    || die "Problem opening $tmpName.dat for writing: $!";
    
for($j=0;$j<scalar(@intensities);$j++){

    for($k=0;$k<5;$k++){
	print  OUT "$intensities[$j][$k+1]\t";  
	if($setmax == 0){
	    ### going to defined a max value based on first tile
	    if($intensities[$j][$k+1] > $max){
		$max = $intensities[$j][$k+1];
	    }	
	}	    

    }
    print OUT "\n";
}
close (OUT);

if($setmax == 0){
    $max = $max * 1.5;
    ## wanted to round to nearest 1000 so will cheat until i have more time
    $max = $max / 1000;
    $max = int($max);
    $max = ($max+1) * 1000;
    $setmax++;
}

draw_graphs($i,$tmpName,$ARGV[1],$title);











