#!/usr/bin/env perl

#### This code plots % error rate curves for 0,1,2,3,4 errors per reads
#### This feature has been requested by Clive
#### The program inputs a realign file
####stores the numbers in a /tmp file to put the graph 
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

use warnings;
use strict;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plot_Error_Curves.pl,v 1.8.8.1 2007/06/11 15:14:46 tc Exp $';
    my $cvs_version='$Revision: 1.8.8.1 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


die "Usage: $0 realign.txt\n"
    unless (@ARGV>=1);


my $length = 0;
my (@array,@results);
my ($i,$j,$count,$lane,$title,$tmpName,$outName,$seq);
my $pathGnuplot="gnuplot";
my $pathConvert="convert";

my $width=24;
my $max = 100;
my $total = 0;
my $name = shift(@ARGV);

# Take output name from command line arguments (recommended usage)
# Recommended usage is that output file name ends in .png, although
# ImageMagick convert can build other image file formats too
if (@ARGV==1)
{
    $outName=shift(@ARGV);
} # if
else
{ # make output name based on assumptions about input name 
  # (not recommended, kept for backwadrds compatibility)
    $outName = "$name";
    $outName =~ s/realign/errors/;
    $outName =~ s/\.txt/\.png/;
    warn "No output filename specified, will send output to $outName\n";
} # else

die if ($outName eq $name);


sub draw_graphs_impulses {
    my ($lane,$tmpName,$outName,$title,$total) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:100];
set ytics 10;
 
    set boxwidth 0.6;
 
set size 1.0, 1.0;
set origin 0.0, 0.0;
set title "$title";
set xlabel "Cycle Number" -1;
set ylabel "% reads with 'x' errors or less";
set key left bottom below
set label "CLUSTERS = $total" at 0,-5,0
   # plot '$tmpName.dat' using 1:2 title '0 errors' with boxes lt 9 lw $width fs solid 0.2;

plot '$tmpName.dat'  using 1:6 title '4 or less' with impulses lt 7 lw $width, '$tmpName.dat' using 1:5 title '3 or less' with impulses lt 2 lw $width,'$tmpName.dat' using 1:4 title '2 or less' with impulses lt 3 lw $width,'$tmpName.dat' using 1:3 title '1 or less' with impulses lt 1 lw $width, '$tmpName.dat' using 1:2 title '0 errors' with impulses lt 9 lw $width; 


#plot '$tmpName.dat' using 1:2 title '0 errors' with boxes lt 9 lw $width fs solid 0.2, '$tmpName.dat' using 1:3 title '1 or less' with boxes lt 1 lw $width  fs solid 0.4,    '$tmpName.dat' using 1:4 title '2 or less' with boxes lt 3 lw $width  fs solid 0.6, '$tmpName.dat' using 1:5 title '3 or less' with boxes lt 2 lw $width  fs solid 0.8,'$tmpName.dat' using 1:6 title '4 or less' with boxes lt 7 lw $width  fs solid 1; 

END
    close (OUT);
    system ("$pathGnuplot $tmpName.txt > $tmpName.ps");
    system ("$pathConvert $tmpName.ps $outName");
    unlink "$tmpName.dat", "$tmpName.txt", "$tmpName.ps";
}





open (IN, $name) 
    || die "Problem opening $name: $!";

while (<IN>)
{
   
     next if ($_ =~ m/^\#/);
#     print "$_";
     chomp;
     
     @array = split(' ',$_);

     # reject multiple hits
     next if($array[2] != 1);

     
     
     $total++;
     $length = length($array[0]);
    
     if($array[4] eq 'R'){
	 $seq=reverse($array[5]);
	 $seq =~ tr/ACGTacgt/TGCAtgca/;
     }
     else {
	 $seq = $array[5];
     }

     $count = 0;
  #   @read = split('',$seq);

     for($i=0;$i<$length;$i++){
	 if(substr($array[0],$i,1) ne substr($seq,$i,1)){
	     $count++;
	 }
	 
	 if($count < 5)
	 {
	     $results[$i][$count]++;
	 } # if count
     } # for
} # while

my $lastCount;
for($i=0;$i<$length;$i++)
{
    $lastCount=0;
    for($j=0;$j<5;$j++)
    {
	$results[$i][$j] = 0 unless (defined($results[$i][$j]));
	$results[$i][$j] +=$lastCount;
	$lastCount=$results[$i][$j];
    } # for j
} # for i

# If no usable alignments in file

unless ($total>0)
{
    warn "No unique alignments in file $name, creating blank file $outName"; 
    open (OUT, ">$outName") 
        || die "Problem creating blank file $outName: $!";
    close (OUT);
    exit (0);
} # unless


	
$title = "$name";
#print "$outName\n";

$tmpName="/tmp/tmp-plot_Error_Curves-$>-$$";
open (OUT, ">$tmpName.dat")
    || die "Problem opening $tmpName.dat for writing: $!";

for($i=0;$i<$length;$i++){
   
    printf OUT "%d\t",$i+1;
#   printf STDERR "%d\t",$i+1;
  
    for($j=0;$j<5;$j++){
	$results[$i][$j] = $results[$i][$j]/$total * 100;
	print OUT "$results[$i][$j]\t";
#	print STDERR "$results[$i][$j]\t";

    }
    print OUT "\n";
#    print STDERR "\n";
}
close (OUT);

$lane = 1;
draw_graphs_impulses($lane,$tmpName,$outName,$title,$total);
