#!/usr/bin/env perl
use warnings;
use strict;

# File: score.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Use output of PhageAlign.cpp to estimate error rate at each cycle.
# This in turn is used to generate log-likelihood scores which
# can be fed back into PhageAlign.cpp. This process is iterated
# to convergence to obtain maximum likelihood error rates.

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: score.pl,v 1.8 2006/08/29 13:39:40 tc Exp $';
    my $cvs_version='$Revision: 1.8 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

# Try to measure error rate by looking at scores
my @a;
my $rev;
my $i;
my $j;
my $k;
my @errs;
my $numUnique=0;
my $x;
my $y;
my $matchScore;
my $seq;

my @bases=qw/A C G T/;
my @readBases=( '.', @bases );
my @allCorrect = qw/ AA CC GG TT /; 
my @allBlank = qw/ .A .C .G .T/;
my @allErrs=qw/ AC AG AT CA CG CT GA GC GT TA TC TG/;
my @allCombs= ( @allCorrect, @allErrs, @allBlank );

my $blankBase='.';
#my %bhash;
# my @numUn;
my $toBase10=1.0/log(10.0);
my $thisErr;
my %allHash;
my @cycHash;
my @cycBases;
#my @cycErrs;
#my @cycBlanks;
my $readAs;
my $shouldBe;
my $numErrs=0;
my $numBases=0;
my $numBlanks=0;


print "#RUN_TIME ". scalar(localtime)."\n";
print "#SOFTWARE_VERSION $id\n";

my %hashSeq;
my %hashBlank;
my @allBases;

while (<STDIN>)
{
    if (/^\#/)
    {
	print; next;
    } #if
    @a=split;

    for ($i=0;$i<length($a[0]);$i++)
    {            
        $allBases[$i]{substr($a[0],$i,1)}++;
    } # for

  
    $hashSeq{$a[0]}++;
    $readAs=$a[0];
    $readAs=~ tr/ACGTacgt/NNNNNNNN/;
    $hashBlank{$readAs}++;

    next if (@a==3);
    die unless (@a>=6);
    next if ($a[2]!=1);
#  next if ($a[1]<24);

    if ($a[4]eq'R') 
    {
	$rev=reverse($a[5]);
	$rev=~tr/ACGT/TGCA/;
    } #if
    else
    {
	$rev=$a[5];
    } #else

#  print "$a[0]\n$rev\n";
    for ($i=0; $i<length($a[0]); $i++)
    {
	$readAs=substr($a[0],$i,1);
	$shouldBe=substr($rev,$i,1);
	$numBases++;
	$cycBases[$i]++;
	$numBlanks+=($readAs eq $blankBase);
	$numErrs+=(($readAs ne $blankBase)&&($readAs ne $shouldBe ));
	$allHash{$readAs.$shouldBe}++;
	$cycHash[$i]{$readAs.$shouldBe}++;
        $cycHash[$i]{$readAs}++;
    } #for
} #while

print "$numBases bases of sequence found\n";
print "$numErrs were errors\n";
print "$numBlanks were blanks\n";

if ($numBases==0)
{
    print STDERR "Warning: did not find any sequences to process\n";
#    exit (0);
} #if
elsif ($numBases==$numBlanks)
{
    print STDERR "Warning: no nonblank bases of sequence to process\n";
    exit (0);
} #elsif

if (($numBases > 0)&&($numBases-$numBlanks>0)) 
{
#    printf "%5.2f percent error rate\n", 100.0*$numErrs/$numBases;
#    printf "%5.2f percent blank rate\n", 100.0*$numBlanks/($numBlanks+$numBases);
    printf "%5.2f percent error rate\n", 100.0*$numErrs/($numBases-$numBlanks);
    printf "%5.2f percent blank rate\n", 100.0*$numBlanks/($numBases);
}
print "\nBreakdown of errors by cycle\n";

my $blankTotal;
my $correctTotal;
my $errTotal;
my $blankRate;
my $errorRate;

print "Cycle:\tErr pc:\tBlank pc:\n";

for ($i=0; $i<@cycBases; $i++)
{
    $blankTotal=0;
    $correctTotal=0;
    $errTotal=0;
    foreach(@allBlank)
    {
	$blankTotal+=$cycHash[$i]{$_}
	if defined ($cycHash[$i]{$_});
    } #foreach
    foreach(@allCorrect)
    {
	$correctTotal+=$cycHash[$i]{$_}
	if defined ($cycHash[$i]{$_});
    } #foreach
    foreach(@allErrs)
    {
	$errTotal+=$cycHash[$i]{$_}
	if defined ($cycHash[$i]{$_});
    } #foreach

    # NB error rate is defined as (#errors) / (#non-blank bases)
    # blank rate is defined as (#blanks) / (#all bases)

    $errorRate=0;
    $errorRate=100.0*$errTotal/($correctTotal+$errTotal)
	unless ($correctTotal+$errTotal==0);
    $blankRate=0;
    $blankRate=100.0*$blankTotal/($correctTotal+$errTotal+$blankTotal)
	unless ($correctTotal+$errTotal+$blankTotal==0);

    printf "%d\t%5.2f\t%5.2f\n", $i+1, 
    $errorRate, $blankRate;
#    print $i+1, " $correctTotal $errTotal $blankTotal\n";

} #for i

my $val;
my $tot;

printf ("\nError rate relative to reference base (including blanks)");
printf ("\n(Given a reference base, what is it sequenced as?)\n\n");

print "Really:\tRead as:\n";
foreach $i (@readBases) { print "\t$i pc" };
print "\n";

foreach $i (@bases)
{
    print "$i";
    $tot=0;
    foreach $j (@readBases)
    {
        $allHash{$j.$i}=0 unless defined($allHash{$j.$i});
	$val=$allHash{$j.$i};
	$tot+=$val;
    } #foreach $j
    foreach $j (@readBases)
    {
	if ($tot==0)
	{
	    $val=0;
	} #if
	else
	{
	    $val=100.0*$allHash{$j.$i}/$tot;
	} #else
	printf "\t%5.2f", $val;
    } #foreach $j
    print "\n";
} #foreach $i

printf ("\nError rate relative to reference base (excluding blanks)");
printf ("\n(Given a reference base, what is it sequenced as?)\n\n");

print "Really:\tRead as:\n";
foreach $i (@bases) { print "\t$i pc" };
print "\n";

foreach $i (@bases)
{
    print "$i";
    $tot=0;
    foreach $j (@bases)
    {
	$val=$allHash{$j.$i};
	$val=0 unless (defined($val));
	$tot+=$val;
    } #foreach $j
    foreach $j (@bases)
    {
	if ($tot==0)
	{
	    $val=0;
	} #if
	else
	{
	    $val=100.0*$allHash{$j.$i}/$tot;
	} #else
	printf "\t%5.2f", $val;
    } #foreach $j
    print "\n";
} #foreach $i

printf ("\nError rate relative to sequenced base");
printf ("\n(Given a sequenced base, what was it really?)\n\n");
print "Read As:\tReally:\n";
foreach $i (@bases) { print "\t$i pc" };
print "\n";

foreach $i (@readBases)
{
    print "$i";
    $tot=0;
    foreach $j (@bases)
    {
        $allHash{$i.$j}=0 unless defined($allHash{$i.$j});
	$val=$allHash{$i.$j};
#	$val=0 unless (defined($val));
	$tot+=$val;
    } #foreach $j
    foreach $j (@bases)
    {
	if ($tot==0)
	{
	    $val=0;
	} #if
	else
	{
	    $val=100.0*$allHash{$i.$j}/$tot;
	} #else
	printf "\t%5.2f", $val;
    } #foreach $j
    print "\n";
} #foreach $i

printf ("\nBreakdown of errors by nucleotide\n\n");
print "Read As:\tReally:\n";
foreach $i (@bases) { print "\t$i pc" };
print "\n";

foreach $i (@readBases)
{
    print "$i";
#    $tot=0;
#   foreach $j (@bases)
#    {
#	$val=$allHash{$i.$j};
#	$val=0 unless (defined($val));
#	$tot+=$val;
#    } #foreach $j
    foreach $j (@bases)
    {
        $val=$allHash{$i.$j};
        $val=0 unless (defined($val));
        printf "\t$val";
    } #foreach $j
    print "\n";
} #foreach $i



print "\nFull breakdown of errors by cycle and nucleotide\n";
print "Cycle:\tRead As:\tReally:\n\t";
foreach $i (@bases) { print "\t$i ct" };
print "\n";
for ($i=0; $i<@cycBases; $i++)
{
    foreach $readAs (@readBases)
    {
	printf "%d\t%s", $i+1, $readAs;
	foreach $shouldBe (@bases)
	{
	    $cycHash[$i]{$readAs.$shouldBe}=0
                unless defined ($cycHash[$i]{$readAs.$shouldBe});
	    $val=$cycHash[$i]{$readAs.$shouldBe};
#	    $val=0 unless (defined($val));
	    die unless (defined($val));
	    printf "\t$val";
	} #foreach
	print "\n";
    } #foreach
} #for i

print "\nError rate relative to reference by cycle/nucleotide\n";
print "Cycle:\tRead As:\tReally:\n\t";
foreach $i (@bases) { print "\t$i ct" };
print "\n";

for ($i=0; $i<@cycBases; $i++)
{
    foreach $readAs (@readBases)
    {
	printf "@%d\t%s", $i+1, $readAs;
	foreach $shouldBe (@bases)
	{
            $tot=0;
            map 
            { $tot+=$cycHash[$i]{$_.$shouldBe} } @readBases;
	    $val=$cycHash[$i]{$readAs.$shouldBe};
#	    $val=0 unless (defined($val));
#	    die unless (defined($val));
            $val/=$tot unless ($tot==0);
	    printf "\t%6.3f", $val;
	} #foreach
	print "\n";
    } #foreach
} #for i

my $thisEntropy;
my $thisProb;
my $priorEntropy=2;
my $toBase2=1.0/log(2.0);
my $thisInfo; 
my $thisAligned;
my $thisTotal;
my $infoSoFar;
my $alignedSoFar;
my $totalSoFar;

print "\nInformation Content By Cycle\n";
print "\tBases this cycle\t\tBases so far\n";
print "Cycle:\tEquiv info:\tAlign:\tTotal:\tEquiv info:\tAlign:\tTotal:\n";

for ($i=0; $i<length($a[0]); $i++)
{
    $thisInfo=0;
    for $j (@bases)
    { # work out entropy contribution for letter $j
        $allBases[$i]{$j}=0 unless defined($allBases[$i]{$j});
     #   $numRef->[$i]+=$allBases[$i]{$j};
        
        next unless 
            (defined ($cycHash[$i]{$j})&&($cycHash[$i]{$j}!=0));
        $thisEntropy=0;
        for $k (@bases)
        { # work out entropy of one instance of letter $j
            next unless 
                (defined ($cycHash[$i]{$j.$k})&&($cycHash[$i]{$j.$k}!=0));
            $thisProb=$cycHash[$i]{$j.$k}/$cycHash[$i]{$j};
            $thisEntropy+=$thisProb*log($thisProb);
#                print "$thisProb\n";
        } # for k
        $thisEntropy*=$toBase2; # convert units to bits
        $thisEntropy+=$priorEntropy; # convert to an info gain
        $thisEntropy/=2.0; # convert units to bases from bits
#            print "$i $j $thisEntropy\n";
        $thisInfo+=$cycHash[$i]{$j}*$thisEntropy;
    } # for j

    $thisAligned=0;
    $thisTotal=0;

    for $j (@bases)
    {
	$thisAligned+=$cycHash[$i]{$j}
	if (defined ($cycHash[$i]{$j}));
	$thisTotal+=$allBases[$i]{$j}
	if (defined ($allBases[$i]{$j}));
    } # for $j


#    $thisAligned
#        =$cycHash[$i]{A}+$cycHash[$i]{C}+$cycHash[$i]{G}+$cycHash[$i]{T};
#    $thisTotal
#        =$allBases[$i]{A}+$allBases[$i]{C}+$allBases[$i]{G}+$allBases[$i]{T};

    $infoSoFar+=$thisInfo;
    $alignedSoFar+=$thisAligned;
    $totalSoFar+=$thisTotal;

    printf "~%d\t%8.2f\t%d\t%d\t%8.2f\t%d\t%d\n",
    $i+1, $thisInfo,
    $thisAligned,
    $thisTotal,
    $infoSoFar,
    $alignedSoFar,
    $totalSoFar;

} # for i   



# Count most common words and most common gap distribution
my $maxNumToList=30;
my $maxPerLine=6;
my $maxGaps=2;
my $numToList;

my %countNum;
my %countSeq;

foreach $seq ( sort keys %hashSeq)
{
    $i=0;
    while ($seq=~/\./g) { $i++; };
    next if ($i>$maxGaps);

    $countSeq{$hashSeq{$seq}}.="$seq ";
    $countNum{$hashSeq{$seq}}++;
} #foreach $i

$numToList=$maxNumToList;
$numToList=keys %countSeq if ((keys %countSeq)<$maxNumToList);
print "\nThe $numToList most common words with $maxGaps blanks or less:\n";
print "Ranking\tOccurrences\tWords\n";

$j=0;
foreach $i ( sort {$b<=>$a} keys %countSeq )
{
    ++$j; last if ($j>30);
    if ($countNum{$i}>$maxPerLine)
    {
	print "$j\t$i\t$countNum{$i} sequences\n";
    } #if
    else
    {
	print "$j\t$i\t$countSeq{$i}\n";
    } #else
} #foreach $seq

# Compute patterns of blank/non blank bases

%countNum=();
%countSeq=();

foreach $seq ( sort keys %hashBlank)
{
    $countSeq{$hashBlank{$seq}}.="$seq ";
    $countNum{$hashBlank{$seq}}++;
} #foreach $i


$numToList=$maxNumToList;
$numToList=keys %countSeq if ((keys %countSeq)<$maxNumToList);
print "\nThe $numToList most common blank patterns (N=any nonblank character)";
print "\nRanking\tOccurrences\tWords\n";

$j=0;
foreach $i ( sort {$b<=>$a} keys %countSeq )
{
    ++$j; last if ($j>30);
    if ($countNum{$i}>6)
    {
	print "$j\t$i\t$countNum{$i} sequences\n";
    } #if
    else
    {
	print "$j\t$i\t$countSeq{$i}\n";
    } #else
} #foreach $seq





# score you would get if each outcome is equiprobable
# give this score in event of no information
# (maybe better to average scores from other cycles)
my $neutralScore=int(100*$toBase10*log(1/(4-1)));


print "\nLog likelihood scores\n";
print "Cycle:\tRead As:\tReally:\n\t";
foreach $i (@bases) { print "\t$i" };
print "\n";
for ($i=0; $i<@cycBases; $i++)
{
    foreach $readAs (@readBases)
    {
	printf ">%d\t%s", $i+1, $readAs;

	$tot=0;
	foreach $shouldBe (@bases)
	{
	    $val=$cycHash[$i]{$readAs.$shouldBe};
	    $val=0 unless (defined($val));
	    $tot+=$val;
	} #foreach
	foreach $shouldBe (@bases)
	{
	    $val=$cycHash[$i]{$readAs.$shouldBe};
	    $val=0 unless (defined($val));
	    $matchScore=$neutralScore;
	    $matchScore=int(100*$toBase10*log($val/($tot-$val)))
		unless (($tot==$val)||($val==0));
	    print "\t$matchScore";
	} #foreach
	print "\n";
    } #foreach
} #for i


