#!/usr/bin/env python
#
# Copyright (c) 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import re, sys, os, getopt, matrix

"""Utility to read a number of Phoenix matrix files and convert them
to a format with a single line per matrix, which consists of lane,
tile and the 16 matrix elements."""

def usage():
    print "Usage: %s <filename> [<filename> ...]" % sys.argv[0]
    
if __name__ == "__main__":

    try:
        (options, args) = getopt.gnu_getopt(sys.argv[1:], "",
                                            ["plot", "flat" ])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    if (len(args) == 0):
        usage()

    fname = re.compile("\w*_(\d)_(\d*)_[\d_\w]*.txt")
    for matrixfile in args:
        fn = os.path.split(matrixfile)[1]
        mtch = fname.match(fn)
        if mtch:
            lane = int(mtch.group(1))
            tile = int(mtch.group(2))
        else:
            print "Failed to parse matrix filename %s!\n" % fn
            sys.exit(3)
        matrix = matrix.read_matrix(matrixfile)
        if not matrix or len(matrix) != 4:
            matrix = [ [0.] *4 for i in range(4)]
        str = ""
        for i in range(4):
            for j in range(4):
                str += "%4.2f" % matrix[i][j]
                if (j < 3):
                    str += " "
            if (i < 3):
                str += "\t"

        print "%i\t%i\t%s" % (lane, tile, str)
        
