#!/usr/bin/env python
#
# Time-stamp: <26 Jun 2006 16:42 BST parse_offset_scaling.py km>
# Copyright (c)  2005, 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import os, sys, glob, string, types
from xml.sax import make_parser, ContentHandler
import XMLSimple

"""Parse the image analysis XML QC files for image offsets and scales and
print them in one line per image."""
#
# Two alternative versions exist: A Sax based XML parser,
# plus the XML parser imitating the Perl XML::Simple module.
# 

def usage():
    print "Usage: %s <XML file> [<XML file> ...]" % sys.argv[0]
    sys.exit(0)


class QCMHandler(ContentHandler):
    def __init__(self):
        self.simpletag = ""
        self.aligntag = ""
        self.inalign = 0
        self.inregion = 0
    def startElement(self, name, attrs):
        if self.inregion:
            return
        if name == "QC":
            self.images = []
        if name == "Image":
            self.images.append({})
        if name in [ "ChannelNumber", "TileNumber",
                     "CycleNumber", "Deblocked"]:
            self.simpletag = name
            self.simplecontent = ""
        if self.inalign and (name in ["OffsetX", "OffsetY",
                                      "StretchX", "StretchY"]):
            self.aligntag = name
            self.aligncontent = ""
        if name == "Alignment":
            self.inalign = 1
            self.alignment = {}
        if name == "ImageRegion":
            self.inregion = 1
            
    def endElement(self, name):
        if name in [ "ChannelNumber", "TileNumber",
                     "CycleNumber", "Deblocked"]:
            if self.simpletag == name:
                self.images[-1][name] = int(self.simplecontent)
            self.simpletag = ""
        if name == "QC":
            pass
            #print self.images
        if name == "Alignment":
            if not self.images[-1].has_key("Alignment"):
                self.images[-1]["Alignment"] = []
            self.images[-1]["Alignment"].append(self.alignment.copy())
            inalignment = 0
            #print name, self.images, self.alignment
        if self.inalign and not self.inregion and \
               (name in ["OffsetX", "OffsetY",
                         "StretchX", "StretchY"]):
            self.alignment[name] = float(self.aligncontent)
            self.aligntag = ""
        if name == "ImageRegion":
            self.inregion = 0
    def characters(self, ch):
        if self.inregion:
            return
        if self.simpletag:
            self.simplecontent += ch
        if self.inalign and self.aligntag:
            self.aligncontent += ch


if __name__ == "__main__":
    if len(sys.argv) < 2:
        usage()
    files = sys.argv[1:]

    # --- SAX version
    handler = QCMHandler()
    saxparser = make_parser()
    saxparser.setContentHandler(handler)
    # --- SAX version
    
    for file in files:

        # --- !SAX version
        #tag, data = XMLSimple.xml_parse_lines(open(file).readlines())
        #images = data['Image']
        # --- SAX version
        try:
            saxparser.parse(open(file))
            images = handler.images
        except:
            sys.stderr.write("Warning: Could not parse %s; skipping\n" %
                             file)
            continue
        # --- SAX version

        #print images
        alignments = []
        for i in range(len(images)):
            image = images[i]
            if not image.has_key('Alignment'):
                continue
            if (type(image['Alignment']) == types.ListType):

                alignments.append(image['Alignment'][-1])
            else:
                alignments.append(image['Alignment'])
            #print image['Alignment']
        if not alignments:
            continue

        for i in range(len(images)):
            image = images[i]
            lane = image['ChannelNumber']
            tile = image['TileNumber']
            cycle = image['CycleNumber']
            deblock = image['Deblocked']
            if cycle <= 1:
                continue
                
            print "%1i %3i %2i %1i %1i" % (lane, tile, cycle, i, deblock),
            print "%6.2f %6.2f %8.5f %8.5f" % \
                  (alignments[i]['OffsetX'], alignments[i]['OffsetY'],
                   alignments[i]['StretchX'], alignments[i]['StretchY']),
            
            print "%8.5f %8.5f" % \
                   (alignments[i]['StretchX'] - alignments[0]['StretchX'],
                    alignments[i]['StretchY'] - alignments[0]['StretchY']),
            print "%6.2f %6.2f" % \
                  (alignments[i]['OffsetX'] - alignments[0]['OffsetX'],
                   alignments[i]['OffsetY'] - alignments[0]['OffsetY']),
            print "%6.2f %6.2f" % \
                  (alignments[i]['OffsetX'] - alignments[1]['OffsetX'],
                   alignments[i]['OffsetY'] - alignments[1]['OffsetY']),
            print "%6.2f %6.2f" % \
                  (alignments[i]['OffsetX'] - alignments[2]['OffsetX'],
                   alignments[i]['OffsetY'] - alignments[2]['OffsetY']),
            print "%6.2f %6.2f" % \
                  (alignments[i]['OffsetX'] - alignments[3]['OffsetX'],
                   alignments[i]['OffsetY'] - alignments[3]['OffsetY'])
