/*
PROJECT: IMPALA (Inexact Matching Program ALlowing Ambiguity)
MODULE: GlobalUtilities.cpp
AUTHOR: A. J. Cox

 * Copyright (c) 2003-2006 Solexa
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <iostream>
#include <fstream>
#include <sys/resource.h>
#include <GlobalUtilities.h>
#include <ctime>
#include <cstdlib>
#include <cstdio>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string>
#include <dirent.h>


// Identifier to embed in source and binary files
static char* global_id="@(#) $Id: GlobalUtilities.cpp,v 1.6 2006/11/06 16:37:01 tc Exp $";

// GlobalUtilities.cpp
// Common definitions and generally useful functions go here

void printWord( Word w, int l )
{
     for ( int i(2*(l-1)) ; i>=0 ; i-=2 )
       cout << baseNames[ (w >> i)&0x3 ];
} // ~void printWord( Word w, int l )

char reverseComp( const char c)
{
  return (c=='A') 
    ? 'T' 
    : ( (c=='T') 
	? 'A' 
	: ( (c=='C') 
	    ? 'G'
	    : ( ( c=='G') ? 'C' : '!')));
} // ~reverseComp;

const char* getPrefixString( Word w, uint prefixLength )
{
  static char buf[10]; 
  char* pNext(buf+prefixLength);
  *pNext='\0';
  do
  {
    *(--pNext)=baseNames[w&0x3];
    w >>= numBitsPerBase;
  } while (pNext!=buf);

  return buf;
} // ~getPrefixString( Word w, uint prefixLength )


Timer::Timer( void ) 
{
  if (gettimeofday(&lastTime_,NULL)!=0) exit(-1);
  if (getrusage(RUSAGE_SELF, &lastUsage_)!=0) exit (-1);
} // ~Timer::Timer( void ) 


ostream& Timer::print( ostream& os )
{
  if (gettimeofday(&thisTime_,NULL)!=0) exit(-1);
  if (getrusage(RUSAGE_SELF, &thisUsage_)!=0) exit (-1);
  
  static double elapsedActual, elapsedUser, elapsedSystem;

  elapsedUser = 
    thisUsage_.ru_utime.tv_sec-lastUsage_.ru_utime.tv_sec
  + (thisUsage_.ru_utime.tv_usec-(double)lastUsage_.ru_utime.tv_usec)/1000000;

  elapsedSystem = 
    thisUsage_.ru_stime.tv_sec-lastUsage_.ru_stime.tv_sec
  + (thisUsage_.ru_stime.tv_usec-(double)lastUsage_.ru_stime.tv_usec)/1000000;

  elapsedActual = 
    thisTime_.tv_sec-lastTime_.tv_sec
  + (thisTime_.tv_usec-(double)lastTime_.tv_usec)/1000000;

  os << "User: " << elapsedUser  
     << "s System: " << elapsedSystem  
     << "s Actual: " << elapsedActual  
     << "s Efficiency: " 
     << ( (elapsedActual==0)
	  ?0
	  :((elapsedUser+elapsedSystem)*100.0/elapsedActual) )
     << '\045'; // only way to print '%' liked by both Intel and gcc!
  lastTime_ = thisTime_;
  lastUsage_ = thisUsage_;
  return os;
} // ~ostream& Timer::print( ostream& os )

ostream& operator<<( ostream& os, Timer& timer )
{
  return timer.print(os);
} // ~ostream& operator<<( ostream& os, Timer& timer )

const char* Timer::timeNow( void ) const
{
  time_t tt(time(NULL));
  return ctime(&tt);
} // ~const char* Timer::timeNow( void ) const


// ExpandedTranslationTable function definitions

ExpandedTranslationTable::ExpandedTranslationTable( int oligoLength ) :
  prefixLength_((oligoLength-16)>>1),
  isOddLength_((oligoLength&0x1)!=0),
  reverseShift1_(2*((2*maxBasesPerWord)-oligoLength)),
  reverseShift2_(numBitsPerWord-reverseShift1_)
{
  for (int i(0);i<numPossibleChars;i++)
    for (int j(0);j<numPossibleChars;j++)
    {
      if ((whichBase[i]==nv)||(whichBase[j]==nv))
	t_[(i<<8)+j]=nv;
      else
	t_[(i<<8)+j]=((whichBase[j]<<numBitsPerBase)|whichBase[i]);
    } // ~for 
} // ~ExpandedTranslationTable::ctor

void ExpandedTranslationTable::translate
( const char* buf, Oligo& ol, Oligo& rc ) const
{
  ol.ui[0]=0;
  ol.ui[1]=0;
  if (isOddLength_)
  {
    ol.ui[1]|=whichBase[(*buf++)];
  } // ~if isOddLength_

  ushort* ps((ushort*)buf);

  for (int i(0);i<prefixLength_;++i)
  {
    ol.ui[1]<<=(2*numBitsPerBase);
    ol.ui[1]|=t_[*(ps++)];
  }

  //  ol.ui[1]<<=(2*numBitsPerBase); // 0
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 1
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 2
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 3
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 4
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 5 
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 6 
  ol.ui[0]|=t_[*(ps++)];

  ol.ui[0]<<=(2*numBitsPerBase); // 7
  ol.ui[0]|=t_[*(ps++)];


  rc.uc[0]=reverseChar[ol.uc[7]];
  rc.uc[1]=reverseChar[ol.uc[6]];
  rc.uc[2]=reverseChar[ol.uc[5]];
  rc.uc[3]=reverseChar[ol.uc[4]];
  rc.uc[4]=reverseChar[ol.uc[3]];
  rc.uc[5]=reverseChar[ol.uc[2]];
  rc.uc[6]=reverseChar[ol.uc[1]];
  rc.uc[7]=reverseChar[ol.uc[0]];

  rc.ui[0]>>=reverseShift1_;
  rc.ui[0]|=(rc.ui[1]<<reverseShift2_);
  rc.ui[1]>>=reverseShift1_;


} // ~ExpandedTranslationTable::translate( char* buf, Word ol[2] )

FileReader::FileReader( const char* fileName )
  : seqFileName_(fileName), vldFileName_(fileName)
{
  seqFileName_ += (string) ".2bpb";
  vldFileName_ += (string) ".vld";

  int fd;

  char* pChar;

  // memory map sequence file
  fd = open(seqFileName_.c_str(), O_RDONLY, S_IRUSR );
  if (fd==-1)
  {
    cerr << "Error in FileReader: could not open file " 
	 << seqFileName_ << endl;
    exit (1);
  } // ~if
  seqFileSize_ = lseek(fd, 0, 2);
  cerr << "FileReader: opened file " << seqFileName_ << " of size "
       << seqFileSize_ << " bytes." << endl;

  pMapSeq_=mmap(0, seqFileSize_, PROT_READ, MAP_SHARED, fd, 0);
  if (pMapSeq_==NULL)
  {
    cerr << "Error in FileReader: could not memory map file " 
	 << seqFileName_<< endl;
    exit (1);
  } // ~if
  close (fd);

  // memory map validity file
  fd = open(vldFileName_.c_str(), O_RDONLY, S_IRUSR );
  if (fd==-1)
  {
    cerr << "Error in FileReader: could not open file " 
	 << vldFileName_ << endl;
    exit (1);
  } // ~if
  vldFileSize_ = lseek(fd, 0, 2);
  cerr << "squash: opened file " << vldFileName_ << " of size "
       << vldFileSize_ << " bytes." << endl;

  pMapVld_=mmap(0, vldFileSize_, PROT_READ, MAP_SHARED, fd, 0);
  if (pMapVld_==NULL)
  {
    cerr << "Error in unsquash: could not memory map file " 
	 << vldFileName_<< endl;
    exit (1);
  } // ~if
  close (fd);

  // set up pointers for Word

  pChar= (char*) pMapSeq_;
  pStart_= (Word*) pMapSeq_;
  pEnd_ = (Word*)(pChar+seqFileSize_);

  // set up last pointer for valid region

  pChar= (char*) pMapVld_;
  pLastValid_ = (ValidRegion*)(pChar+vldFileSize_);

  cerr << "Comment: ";

  // print out the comment line ...
  while (*pChar != '\n')
  {
    cerr.put(*pChar);
    pChar++;
  } // ~while

  cerr.put('\n');
  // ... and the '\n' afterwards
  //  cout.put(*(pChar++));
  pChar++;
  //  cout << endl; // %%%%%

  // set up 
  pValid_ = (ValidRegion*) pChar;
  //  assert( pValid_!=pLastValid_ );
  //  getNextValidRegion();

  cerr << "finished making FileReader" << endl; // %%%%
} // ~FileReader::FileReader

FileReader::~FileReader( void )
{
  cerr << "FileReader: unmapping " << seqFileSize_ << " bytes of memory" 
       << endl;
  munmap (pMapSeq_, seqFileSize_);
  cerr << "FileReader: unmapping " << vldFileSize_ << " bytes of memory" 
       << endl;
  munmap (pMapVld_, vldFileSize_);

} // ~FileReader::~FileReader

// getOligoSource: given the name of a file of oligo data, returns a pointer
// to an instance of the appropriate subclass of OligoSource.
OligoSource* OligoSource::getOligoSource( const char* fileName )
{
  DIR* pDir;
  if ((pDir=opendir(fileName))!=NULL)
  {
    closedir(pDir);
    cerr << "getOligoSource: assuming " << fileName 
	 << " is a directory of oligo files" << endl;
    return new OligoSourceDirectory(fileName);
  } // ~if
  else
  {
    // makes more sense to delegate rest to getOligoSource until (if) we want 
    // to get oligos from non-file sources such as database clients
    return OligoSourceFile::getOligoSourceFile( fileName );
  } // ~else
} // ~getOligoSource


// getOligoSource: given the name of a file of oligo data, returns a pointer
// to an instance of the appropriate subclass of OligoSourceFile.
// Examines first character of file:
// If a '>', assumes a fasta file and returns an OligoSourceFasta
// If valid sequence character, assumes raw sequence and return OligoSourceRaw
// else exit with an error
OligoSourceFile* OligoSourceFile::getOligoSourceFile( const char* fileName )
{
  //  ifstream oligoFile(fileName);
  // if (oligoFile.fail())
  //  {
  //   cerr << "Error in getOligoSourceFile: could not open file " 
  //	 << fileName << endl;
  //   exit (1);
  //  } // ~if
  //  char firstChar(oligoFile.get());
  //  oligoFile.close();
  FILE* pFile=fopen(fileName, "r");  
  if (pFile==NULL) 
  {
    cerr << "Error in getOligoSourceFile: could not open file "
         << fileName << endl;
    exit (1);
  } // ~if
  int outInt;
  if ((outInt=fgetc(pFile))==EOF)
  {
    cerr << "Error in getOligoSourceFile: could not read first char from file "
         << fileName << endl;
    exit (0); // Allow make pipeline to carry on... Zero exit code.
  } // ~if

  fclose(pFile);
  char firstChar((char)outInt);

  if (firstChar=='>')
  {
    cerr << "getOligoSourceFile: detected fasta format oligo file" << endl;
    return new OligoSourceFasta(fileName);
  } // ~if
  else if ((whichBase[firstChar]!=nv)||isBlank(firstChar))
  {
    cerr 
      << "getOligoSourceFile: detected raw sequence format oligo file" << endl;
    return new OligoSourceRaw(fileName);
  } // ~else if
  else if ((firstChar==' ')||(firstChar=='-'))
  {
    cerr  << "getOligoSourceFile: detected quality value format oligo file" 
          << endl;
    return new OligoSourceScore(fileName);
  }
  else if ((firstChar=='#')||(isdigit(firstChar)!=0))
  {
    cerr 
      << "getOligoSourceFile: detected Goat format oligo file" << endl;
    return OligoSourceGoat::getOligoSourceGoat(fileName);
  }
  else
  {
    cerr << "Error in getOligoSourceFile: could not recognize format of file " 
	 << fileName << endl;
    exit (1);
  } // ~else
  return NULL; // compiler too stupid to realize can never get here
} // ~OligoSourceFile* getOligoSourceFile( const char* fileName )

// **
// ** Function definitions for OligoSourceGoat and subclasses
// **

OligoSourceGoat* OligoSourceGoat::getOligoSourceGoat
( const char* fileName )
{

  FILE* pFile=fopen(fileName, "r");
  if (pFile==NULL) 
  {
    cerr << "Error in getOligoSourceGoat: could not open file "
         << fileName << endl;
    exit (1);
  } // ~if

  char buf1[maxLineLength], buf2[maxLineLength];
  while(1)
  {
    if (fgets(buf1, maxLineLength, pFile )==NULL) return NULL;
    if (buf1[0]!='#') break;
  } // ~while
  fclose(pFile);
  int temp;
  sscanf(buf1, "%d %d %d %s", &temp, &temp, &temp, buf2);
  uint i, numBases(0),numDigits(0);
  for (i=0; i<strlen(buf2);i++)
  {
    numBases+=((whichBase[buf2[i]]!=nv)||isBlank(buf2[i]));
    if (isdigit(buf2[i])!=0) numDigits++;
  } // ~for i 

  if ((numBases==0)&&(sscanf(buf2, "%d", &temp)))
  {
    cerr << "getOligoSourceGoat: detected new format Goat file\n";
    return new OligoSourceGoatNew(fileName);
  } // ~if
  else if ((numBases==strlen(buf2))&&(numDigits==0))
  {
    cerr << "getOligoSourceGoat: detected old format Goat file\n";
    return new OligoSourceGoatOld(fileName);
  } // ~else if
  else
  {    
    cerr << "error in getOligoSourceGoat: could not recognize format of file "
	 << fileName << endl;
    exit(1);
  } // ~else
} // ~OligoSourceGoat* OligoSourceGoat::getOligoSourceGoat

const char* OligoSourceGoat::getNextOligo( void )
{
  while(1)
  {
    if (fgets(nameBuf_, maxLineLength, pFile_ )==NULL) return NULL;
    if (nameBuf_[0]!='#') break;
  } // ~while

  parseLine();

  //  char* p(oligoBuf_);
  //  const int sz(strlen(oligoBuf_));
  //  for (int i(0);i<sz;i+=2)
  //  { 
  //   *p++=oligoBuf_[i];
  //  } // ~for
  // *p='\0';
  return oligoBuf_;
} // ~OligoSourceGoat::getNextOligo( void )

const char* OligoSourceGoatOld::getLastName( void )
{
  sprintf(nameBuf_, ">%d-%d-%d", nums_[0],nums_[1],nums_[2]);
  return nameBuf_;
} // ~OligoSourceGoatOld::getLastName( void )

const char* OligoSourceGoatNew::getLastName( void )
{
  sprintf(nameBuf_, ">%d-%d-%d-%d", nums_[0],nums_[1],nums_[2],nums_[3]);
  return nameBuf_;
} // ~OligoSourceGoatNew::getLastName( void )

// ** 
// ** Function definitions for OligoSourceScore
// ** 

// Returns pointer to ASCII sequence of next oligo, or null if at end
const char* OligoSourceScore::getNextOligo( void )
{
    if (fgets(oligoBuf_, maxLineLength, pFile_ )==NULL) return NULL;
    const char* p(oligoBuf_);
    int i(0);
    int temp[numDifferentBases];
    // Fill in score values
    while (1)
    {
      if (sscanf( p, "%d%d%d%d\t", 
              &temp[0], 
              &temp[1],  
              &temp[2], 
              &temp[3] )==EOF) break;
      for (int j(0);j<numDifferentBases;j++)
        scoreTable_[j][i]=(BaseScore)temp[j];
      // TBD check for overflow in above loop
      //      cout << i << ": " << qualityStore_[0][i] << " "<< qualityStore_[1][i] << " "<< qualityStore_[2][i] << " "<< qualityStore_[3][i] << endl;
      i++;

      if ((p=strchr(p,'\t'))==NULL) break;
      // scan past the deblock cycle stuff
      // no ... don't!!
      //      if ((p=strchr(++p,'\t'))==NULL) break;
      ++p;
      //      if (p!=NULL) { p++; p=strchr(p,'\t'); cout << "scan" << endl;}
      //  if (p!=NULL) p++;
    } // ~while

    // Call consensus based on highest score - TBD handle "dead heats"
    BaseScore maxQual, thisQual;
    int maxPos;
    for (int j(0);j<i;j++)
    {
      maxQual=scoreTable_[0][j];
      maxPos=0;
      for (int k(1); k<numDifferentBases; k++)
      {
        thisQual=scoreTable_[k][j];
        if (thisQual>maxQual)
        {
          maxPos=k;
          maxQual=thisQual;
        }
        else if (thisQual==maxQual)
        {
          maxPos=-1;
        }
        if (scoreTable_[k][j]>maxQual)
        {
          maxQual=scoreTable_[k][j];
          maxPos=k;
        } // ~if
      } // ~for k
      oligoBuf_[j]=((maxPos==-1)?'.':baseNames[maxPos]);
    } // ~for j
#ifdef OLD
    for (int j(0);j<i;j++)
    {
      maxQual=scoreTable_[0][j];
      maxPos=0;
      for (int k(1); k<numDifferentBases; k++)
      {
        if (scoreTable_[k][j]>maxQual)
        {
          maxQual=scoreTable_[k][j];
          maxPos=k;
        } // ~if
      } // ~for k
      oligoBuf_[j]=baseNames[maxPos];
    } // ~for j
#endif
    oligoBuf_[i]='\0';
    return oligoBuf_;
} // ~OligoSourceScore::getNextOligo( void )


// **
// ** Function definitions for OligoSourceDirectory 
// **

OligoSourceDirectory::OligoSourceDirectory( const char* dirName ) :
  pSource_(NULL)
{
  DIR* pDir;
  if (!(pDir=opendir(dirName)))
  {
    cerr << "Error: failed to open directory " << dirName << endl;
    exit (1);
  } // ~if

  dirent* dirEntry;
  while ((dirEntry=readdir(pDir)))
  {
    if (dirEntry->d_name[0]!='.')
    {
      fileNames_.push_back
	( (string)dirName
	  +(string)"/"
	  +(string)dirEntry->d_name);
    } // ~if
  } // ~while
  closedir (pDir);
  if (fileNames_.size()==0)
  {
    cerr << "Error: directory " << dirName << " is empty" << endl;
    exit (1);
  } // ~if

  pName_=fileNames_.begin();
  pSource_=getOligoSource(pName_->c_str());
  format_=pSource_->useAssembleFormat();
} // ~ctor


  // Returns pointer to ASCII sequence of next oligo, or null if at end
const char* OligoSourceDirectory::getNextOligo( void )
{
  const char* p;
  if (pName_==fileNames_.end()) return NULL;
  if ((p=pSource_->getNextOligo())!=NULL) 
    return p;
  else
  {
    if (++pName_==fileNames_.end()) return NULL;
    delete pSource_;
    pSource_=getOligoSource(pName_->c_str());
    // just calling pSource_->getNextOligo here mucks things up
    // if for any reason the source is empty 
    return getNextOligo();
  } // ~else

} // ~OligoSourceDirectory::getNextOligo( void )

// Returns pointer to ASCII name of last oligo read
const char* OligoSourceDirectory::getLastName( void )
{
  if (pName_==fileNames_.end()) return NULL;
  return pSource_->getLastName();
} // ~OligoSourceDirectory::getLastName( void )

// Rewind - next oligo read will be first in list
void OligoSourceDirectory::rewind ( void )
{
  delete pSource_;
  pName_=fileNames_.begin();
  pSource_=getOligoSource(pName_->c_str());
} // ~OligoSourceDirectory::rewind( void )


// ScoreSource subclass function definitions

BaseScore OligoSourceScore::getScore( uint baseNum, uint cycle ) const
{
  assert( baseNum<numDifferentBases ); 
  assert( cycle < maxSeqSize );
  return scoreTable_[baseNum][cycle];
} // ~OligoSourceScore::getScore( uint baseNum, uint cycle ) const


BaseScore OligoSourceScore::getScore( char base, uint cycle ) const
{
  return getScore( baseCodes[base], cycle );
} // ~OligoSourceScore::getScore( char base, uint cycle ) const



BaseScore ScoreSourceBasic::getScore( char base, uint cycle ) const
{
  assert(cycle<maxSeqSize);
  const char b(oligos_.getLastOligo()[cycle]);
  if (isBlank(b)) return scoreBlank_;
  else if (tolower(base)==tolower(b))
    return scoreMatch_;
  else return scoreMismatch_;
} // ~getScore( char base, uint cycle ) const

BaseScore ScoreSourceBasic::getScore( uint baseNum, uint cycle ) const
{
  assert(baseNum<numDifferentBases);
  return ScoreSourceBasic::getScore( baseChars[baseNum], cycle );
} // ~getScore( uint baseNum, uint cycle ) const


ScoreSourceCycle::ScoreSourceCycle
( const OligoSource& oligos, const char* scoreFile ) :
  oligos_(oligos)
{

    char lineBuf[maxLineLength];
    int scores[numDifferentBases];
    const BaseScore scoreNotSet(-20000);
    char thisChar;
    cerr << "Reading score information from file " << scoreFile
	 << endl;

    for (uint i(0); i<maxSeqSize; i++)
      for (uint j(0); j<numDifferentBases+1; j++)
	for (uint k(0); k<numDifferentBases; k++)
	  scoreTable_[i][j][k]=scoreNotSet;

    FILE* pScore;
    uint i;
    if ((pScore=fopen(scoreFile, "r" ))==NULL)
    {
      cerr << "Error: could not open file "
	   << scoreFile << endl;
      exit (1);
    } // ~if

    while (fgets(lineBuf, maxLineLength, pScore)!=NULL)
    {
      if (lineBuf[0]!='>') continue;
      //      sscanf( lineBuf, ">%i\t%i\t%i", 
      //      &i, &match, &mismatch);
      sscanf( lineBuf, ">%i\t%c\t%i\t%i\t%i\t%i", 
	      &i, &thisChar, scores, scores+1, scores+2, scores+3);
      assert((i>=1)&&(i<=maxSeqSize));
      assert(baseCodes[thisChar]!=nc);

      for (int j(0);j<numDifferentBases;j++)
	scoreTable_[i-1][baseCodes[thisChar]][j]=(int)scores[j];
      //      scoreMatch[i-1]=match;
      //   scoreMismatch[i-1]=mismatch;
    } // ~while

    // check we have found all score table values we need
    //    for (uint i(0); i<seqSize; i++)
    //  for (uint j(0); j<numReadableBases; j++)
    //	for (uint k(0); k<numDifferentBases; k++)
    //	  assert(scoreTable[i][j][k]!=scoreNotSet);

    fclose (pScore);

} //~ScoreSourceCycle::ctor


BaseScore ScoreSourceCycle::getScore( uint baseNum, uint cycle ) const
{
  assert(baseNum<numDifferentBases);
  assert(cycle<maxSeqSize);
  return scoreTable_[cycle][baseCodes[oligos_.getLastOligo()[cycle]]][baseNum];
} // ~ScoreSourceCycle::getScore( uint baseNum, uint cycle ) const


BaseScore ScoreSourceCycle::getScore( char base, uint cycle ) const
{
  return getScore( baseCodes[base], cycle );
} // ~ScoreSourceCycle::score( char base, int cycle ) const






// End of file GlobalUtilities.cpp







