/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <cstdio>
#include <iostream>
#include "QualityTable.h"

int main(int argc, char *argv[]) 
{
    if (argc < 2) {
        printf("Usage: QCalibration <qvalue-file1> [<qvalue-file2> ...]\n");
        exit(0);
    }

    const int no_params = 4;
    const int binsizes[4] = { 25, 11, 11, 22 };
    //const int binsizes[3] = { 25, 10, 18 };
    float stepsize;
    std::vector<std::vector<float> > bin_boundaries;
    int i, j;
    bin_boundaries.resize(no_params);
    QualityTable *table = new QualityTable();

    // first parameter: cycle no
    bin_boundaries[0].push_back(0.5f);
    for (i = 0; i < binsizes[0]-1; i++)
        bin_boundaries[0].push_back(float(i) + 10.5f);
    bin_boundaries[0].push_back(100.f);

    // second parameter: purity score per base
    // TODO: use a sensible binning from 0.5 to 1. instead of 0 to 1.
    stepsize = 0.5f/(binsizes[1]-1);
    bin_boundaries[1].push_back(-stepsize/2.f);
    for (i = 0; i < binsizes[1]-1; i++)
        bin_boundaries[1].push_back(0.5f+float(i)*stepsize + stepsize/2.f);
    bin_boundaries[1].push_back(1.f+stepsize/2.f);
        
    // third parameter: purity score over 12 bases
    stepsize = 0.5f/(binsizes[2]-1);
    bin_boundaries[2].push_back(-stepsize/2.f);
    for (i = 0; i < binsizes[2]-1; i++)
        bin_boundaries[2].push_back(0.5f+float(i)*stepsize + stepsize/2.f);
    bin_boundaries[2].push_back(1.f+stepsize/2.f);
        
    // fourth parameter: uncalibrated quality score
    stepsize = 44./binsizes[3];
    bin_boundaries[3].push_back(-100.f);
    for (i = 0; i < binsizes[3]-1; i++)
        bin_boundaries[3].push_back(-40.f + float(i)*stepsize +0.5);
    bin_boundaries[3].push_back(10.+0.5);

    // add parameters to table
    for (i = 0; i < bin_boundaries.size(); i++)
        table->addParameter(bin_boundaries[i]);

    // fill table with values
    for (i = 1; i < argc; i++) {
        DataSet *data = new DataSet(argv[i]);
        fprintf(stderr, "%i %i %s\n", argc, i, argv[i]);
        if (data->getNumberParameters() != no_params) {
            std::cerr << "Wrong number of parameters: "<< 
                data->getNumberParameters() << std::endl;
            continue;
        }
        table->addData(data);
        delete data;
    }
#if 0
    table->printDataTable();
    table->error_table.create(&(table->data_table));
    table->printErrorTable();
#endif
    table->create();
    table->print();
    delete table;

    return 0;
}
