/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include "ReadParams.h"

DataSet::DataSet() {}

DataSet::DataSet(const char filename[])
{
    read(filename);
}

DataSet::~DataSet() {}

int DataSet::getNumberParameters() const
{
    return values.size();
}

int DataSet::getNumberValues() const
{ 
    if (values.size() > 0)
        return values[0].size();
    return 0;
}

/*
 * Read the data set from a file. See the comments in the header file and the
 * documentation for the data format.
 */
int DataSet::read(const char filename[])
{
    std::string line;
    std::ifstream file(filename);
    
    int   no_params = 0;
    int   count = 0;
    int   allocsize = 100000;
    BaseAlignment read;
    while (getline(file, line)) {
        if (line.size()==0 || line[0]=='#' || line[0]=='>')
            continue;

        // determine number of columns
        if (no_params == 0) {
            std::stringstream ss(line);
            std::string s;
            while (ss >> s)
                no_params++;
            no_params -= 2;  // last two columns hold base information
            values.resize(no_params);
            std::cerr << "Detected " << no_params << " parameters.\n";
        }
        
        // reserve some memory in advance
        if (count % allocsize == 0) {
            int i;
            for (i = 0; i < values.size(); i++)
                values[i].reserve((count/allocsize+1) * allocsize);
        }
        
        std::stringstream ss(line);
        std::vector<std::string>fields;
        std::string s;
        while (ss >> s)
            fields.push_back(s);
        if (fields.size() != no_params+2)
            std::cerr << "Invalid record: " << line << std::endl;
        int i;
        for (i = 0; i < no_params; i++) {
            float value = atof(fields[i].c_str());
            values[i].push_back(value);
        }
        read.base_read    = fields[fields.size()-2].c_str()[0];
        read.base_aligned = fields[fields.size()-1].c_str()[0];
        reads.push_back(read);
    }    
}
    
std::vector<float> *DataSet::getParameterValues(int idx)
{
    return & (values[idx]);
}

void DataSet::addParameterValues(std::vector<float> &new_values)
{
    values.push_back(new_values);
}

std::vector<std::vector<float> > *DataSet::getParameterValues()
{
    return &values;
} 

std::vector<BaseAlignment> *DataSet::getBaseReads()
{
    return & (reads);
}
