/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

/*
 * This class reads a data set from a file and stores it.
 * The file format consists of a variable number of columns containing the
 * parameters. Each row corresponds to one base-call. The final two columns
 * contain the called and aligned base respectively.
 */
#ifndef READPARAMS_H_

#include <vector>

class BaseAlignment {
public:
    BaseAlignment() : base_read(0), base_aligned(0) {};
    char base_read;
    char base_aligned;
};

class DataSet {
    std::vector<std::vector<float> > values;
    std::vector<BaseAlignment> reads;
public:
    DataSet();
    DataSet(const char filename[]);
    virtual ~DataSet();
    int getNumberParameters() const;
    int getNumberValues() const;
    void addParameterValues(std::vector<float> &new_values);
    int read(const char filename[]);
    std::vector<float> *getParameterValues(int idx);
    std::vector<std::vector<float> > *getParameterValues();
    std::vector<BaseAlignment> *getBaseReads();
};
#endif /*READPARAMS_H_*/
