#!/usr/bin/env python

import sys, os, string, getopt


def readParameters(alignfile, signalfile, prbfile, useBases, PURE_BASES):
    skipColumns = 4
    aligndata =  open(alignfile).readlines()
    signaldata = open(signalfile).readlines()
    prbdata =    open(prbfile).readlines()
    aligndata =  filter(lambda x: x[0] != '#', aligndata)
    signaldata = filter(lambda x: x[0] != '#', signaldata)
    prbdata =    filter(lambda x: x[0] != '#', prbdata)

    if ((len(prbdata) != len(signaldata)) or
        (len(prbdata) != len(aligndata))):
        sys.stderr.write("Wrong number of lines in input files\n")
        sys.exit(1)
    nreads = len(aligndata)
    if nreads == 0:
        return
    length = len(prbdata[0].split())/4
    readlength = len(aligndata[0].split()[0])
    if useBases == "all":
        useBases = 'Y' * length
    if useBases == "odd":
        useBases = 'YN' * min(length/2)
        if length%2:
            useBases += 'Y'
    if len(useBases) < length:
        useBases += "n" * (length - len(useBases))
    useCycles = [ -1 ] * length
    alignCycle = 0
    sequenceCycle = 0
    cycleNumbers = [ 1 ] * length 
    for j in range(length):
        if useBases[j] in ['y', 'Y']:
            if alignCycle < readlength:
                useCycles[j] = alignCycle
            alignCycle += 1
        if useBases[j] in ['y', 'Y', 'n']:
            sequenceCycle += 1
        cycleNumbers[j] = sequenceCycle
    #print useCycles, cycleNumbers

    print "# P PB PB B T"
    for i in range(nreads):
        signal = signaldata[i].split("\t")[skipColumns:]
        prb = prbdata[i].split("\t")
        alignment = aligndata[i].split()
        refseq = "." * readlength
        sequence = alignment[0]
        if (len(alignment) >= 6):
            refseq = alignment[5]
            if alignment[4] == 'R':
                refseq = list(refseq)
                refseq.reverse()
                refseq = string.translate(string.join(refseq, ""),
                                          string.maketrans("ACGT", "TGCA"))

        purity  = [ 1.] * length
        quality = [ 5. ] * length
        purity2 = [ ]
        
        for j in range(length):
            quality[j] = - max(map(int, prb[j].split()))
            signals = map(float, signal[j].split()) 
            signals.sort()
            if signals[-2] <= 0.:
                purity[j] = 1.
            else:
                purity[j] = 1. - signals[-1]/(signals[-1]+signals[-2])
            idx = useCycles[j]
            if idx >= 0 and idx < PURE_BASES:
                purity2.append(purity[j])
           
        for j in range(length):
            idx = useCycles[j]
            ss = '.'
            rs = '.'
            if idx != -1:
                ss = sequence[idx]
                rs = refseq[idx]
            print ("%i\t%4.2f\t%4.2f\t%i\t%c\t%c" %
                   (cycleNumbers[j], purity[j], max(purity2),
                    quality[j], ss, rs))

def usage():
    print "usage: extractQualityParams.py <alignmentfile> <useBases>\n"

if __name__ == "__main__":
    try:
        (options, args) = getopt.gnu_getopt(sys.argv[1:], "",
                                            [ "sig-suffix=", "USE_BASES=",
                                              "PURE_BASES=", "dir=" ])
    except:
        usage()
        sys.exit(1)

    sig_suffix = "_sig2.txt"
    prb_suffix = "_prb.txt"
    PURE_BASES = 12
    USE_BASES ="all"
    
    if len(args) != 1:
        usage()
        sys.exit(1)

    alignfile = args[0]
    if not os.path.exists(alignfile):
        sys.stderr.write("File %s not found\n" % alignfile)
    path, filename = os.path.split(alignfile)
    path = os.path.join(path, "..")

    for o, a in options:
        if o == "--sig-suffix":
            sig_suffix = a
        if o == "--dir":
            path = a
        if o == "--PURE_BASES":
            PURE_BASES = int(a)
        if o == "--USE_BASES":
            USE_BASES = a
            
            
    tile = "_".join(filename.split("_")[:-1])
    signalfile = os.path.join(path, tile+("%s" % sig_suffix))
    prbfile    = os.path.join(path, tile+("%s" % prb_suffix ))
    #print signalfile, prbfile, USE_BASES, PURE_BASES
    if not os.path.exists(signalfile):
        sys.stderr.write("File %s not found\n" % signalfile)
    if not os.path.exists(prbfile):
        sys.stderr.write("File %s not found\n" % prbfile)
    readParameters(alignfile, signalfile, prbfile, USE_BASES, PURE_BASES)
    
